@@ -45,8 +45,15 @@ return {
                     name: 'search',
                     label: '@i18n:objects.config.search',
                     fields: [
-                        'ipasearchrecordslimit',
-                        'ipasearchtimelimit'
+                        {
+                            name: 'ipasearchrecordslimit',
+                            tooltip: '@mc-opt:config_mod:ipasearchtimelimit:doc'
+                        },
+                        {
+                            name: 'ipasearchtimelimit',
+                            measurement_unit: 'seconds',
+                            tooltip: '@mc-opt:config_mod:ipasearchtimelimit:doc'
+                        }
                     ]
                 },
                 {
@@ -105,7 +112,8 @@ return {
                                 { label: '@i18n:authtype.type_radius', value: 'radius' },
                                 { label: '@i18n:authtype.type_otp', value: 'otp' },
                                 { label: '@i18n:authtype.type_pkinit', value: 'pkinit' },
-                                { label: '@i18n:authtype.type_hardened', value: 'hardened' }
+                                { label: '@i18n:authtype.type_hardened', value: 'hardened' },
+                                { label: '@i18n:authtype.type_idp', value: 'idp' }
                             ],
                             tooltip: {
                                 title: '@i18n:authtype.config_tooltip',
@@ -116,6 +124,10 @@ return {
                             $type: 'checkbox',
                             name: 'ipamigrationenabled'
                         },
+                        {
+                            $type: 'checkbox',
+                            name: 'ipauserdefaultsubordinateid'
+                        },
                         {
                             $type: 'multivalued',
                             name: 'ipauserobjectclasses'
