@@ -210,6 +210,7 @@ return {
                         },
                         {
                             $type: 'certs',
+                            name: 'usercertificate',
                             acl_param: 'usercertificate',
                             acl_result_index: 0,
                             adapter: {
@@ -239,7 +240,8 @@ return {
                                 { label: '@i18n:authtype.type_radius', value: 'radius' },
                                 { label: '@i18n:authtype.type_otp', value: 'otp' },
                                 { label: '@i18n:authtype.type_pkinit', value: 'pkinit' },
-                                { label: '@i18n:authtype.type_hardened', value: 'hardened' }
+                                { label: '@i18n:authtype.type_hardened', value: 'hardened' },
+                                { label: '@i18n:authtype.type_idp', value: 'idp' }
                             ],
                             tooltip: {
                                 title: '@i18n:authtype.user_tooltip',
@@ -256,6 +258,17 @@ return {
                         {
                             name: 'ipatokenradiususername',
                             flags: ['w_if_no_aci']
+                        },
+                        {
+                            $type: 'entity_select',
+                            name: 'ipaidpconfiglink',
+                            flags: ['w_if_no_aci'],
+                            other_entity: 'idp',
+                            other_field: 'cn'
+                        },
+                        {
+                            name: 'ipaidpsub',
+                            flags: ['w_if_no_aci']
                         }
                     ]
                 },
@@ -306,6 +319,11 @@ return {
                             label: '@mo-param:pwpolicy:krbpwdlockoutduration:label',
                             read_only: true,
                             measurement_unit: 'seconds'
+                        },
+                        {
+                            name: 'passwordgracelimit',
+                            label: '@mo-param:pwpolicy:passwordgracelimit:label',
+                            read_only: true
                         }
                     ]
                 },
