@@ -60,7 +60,7 @@ define(['dojo/_base/declare',
         kerberos_msg: "<i class=\"fa fa-info-circle\"></i> To log in with " +
             "<strong>Kerberos</strong>, please make sure you" +
             " have valid tickets (obtainable via kinit) and <a href=" +
-            "'http://${host}/ipa/config/ssbrowser.html'>configured</a>" +
+            "'${protocol}//${host}/ipa/config/ssbrowser.html'>configured</a>" +
             " the browser correctly, then click Login. ",
         cert_msg: "<i class=\"fa fa-info-circle\"></i> To log in with " +
             "<strong>certificate</strong>, please make sure you have valid " +
@@ -546,7 +546,7 @@ define(['dojo/_base/declare',
         },
 
         set_reset_aside_text: function() {
-            this.set('aside', "<p>"+this.otp_info_msg+"<p/>");
+            this.set('aside', "<p>"+this.otp_reset_info_msg+"<p/>");
         },
 
         constructor: function(spec) {
@@ -582,9 +582,9 @@ define(['dojo/_base/declare',
                 this.continue_msg
             );
 
-            this.kerberos_msg = this.kerberos_msg.replace(
-                '${host}', window.location.hostname
-            );
+            this.kerberos_msg = this.kerberos_msg
+                .replace('${protocol}', window.location.protocol)
+                .replace('${host}', window.location.hostname);
 
             this.password_change_complete = text.get(
                 spec.password_change_complete ||
@@ -647,7 +647,7 @@ define(['dojo/_base/declare',
             label: text.get('@i18n:login.password', "Password"),
             placeholder: text.get(
                 '@i18n:login.password_and_otp',
-                'Password or Password+One-Time-Password'
+                'Password or Password+One-Time Password'
             ),
             show_errors: false,
             undo: false
@@ -673,17 +673,6 @@ define(['dojo/_base/declare',
             show_errors: false,
             undo: false
         },
-        {
-            name: 'otp',
-            $type: 'password',
-            label: text.get('@i18n:password.otp', "OTP"),
-            placeholder: text.get(
-                '@i18n:password.otp_long',
-                'One-Time-Password'
-            ),
-            show_errors: false,
-            undo: false
-        },
         {
             name: 'new_password',
             $type: 'password',
@@ -717,6 +706,17 @@ define(['dojo/_base/declare',
             }],
             show_errors: false,
             undo: false
+        },
+        {
+            name: 'otp',
+            $type: 'password',
+            label: text.get('@i18n:password.otp', "OTP"),
+            placeholder: text.get(
+                '@i18n:password.otp_long',
+                'One-Time Password'
+            ),
+            show_errors: false,
+            undo: false
         }
     ];
 
