@@ -207,13 +207,6 @@ class selinuxusermap_add(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
     )
     has_output = (
         output.Output(
@@ -224,8 +217,9 @@ class selinuxusermap_add(Method):
         output.Entry(
             'result',
         ),
-        output.PrimaryKey(
+        output.Output(
             'value',
+            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
@@ -257,20 +251,13 @@ class selinuxusermap_add_host(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
         parameters.Str(
             'host',
             required=False,
             multivalue=True,
             cli_name='hosts',
             label=_(u'member host'),
-            doc=_(u'hosts to add'),
+            doc=_(u'comma-separated list of hosts to add'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -279,7 +266,7 @@ class selinuxusermap_add_host(Method):
             multivalue=True,
             cli_name='hostgroups',
             label=_(u'member host group'),
-            doc=_(u'host groups to add'),
+            doc=_(u'comma-separated list of host groups to add'),
             alwaysask=True,
         ),
     )
@@ -326,20 +313,13 @@ class selinuxusermap_add_user(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
         parameters.Str(
             'user',
             required=False,
             multivalue=True,
             cli_name='users',
             label=_(u'member user'),
-            doc=_(u'users to add'),
+            doc=_(u'comma-separated list of users to add'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -348,7 +328,7 @@ class selinuxusermap_add_user(Method):
             multivalue=True,
             cli_name='groups',
             label=_(u'member group'),
-            doc=_(u'groups to add'),
+            doc=_(u'comma-separated list of groups to add'),
             alwaysask=True,
         ),
     )
@@ -400,8 +380,10 @@ class selinuxusermap_del(Method):
             dict,
             doc=_(u'List of deletions that failed'),
         ),
-        output.ListOfPrimaryKeys(
+        output.Output(
             'value',
+            unicode,
+            doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
 
@@ -417,8 +399,6 @@ class selinuxusermap_disable(Method):
             label=_(u'Rule name'),
         ),
     )
-    takes_options = (
-    )
     has_output = (
         output.Output(
             'summary',
@@ -430,8 +410,9 @@ class selinuxusermap_disable(Method):
             bool,
             doc=_(u'True means the operation was successful'),
         ),
-        output.PrimaryKey(
+        output.Output(
             'value',
+            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
@@ -448,8 +429,6 @@ class selinuxusermap_enable(Method):
             label=_(u'Rule name'),
         ),
     )
-    takes_options = (
-    )
     has_output = (
         output.Output(
             'summary',
@@ -461,8 +440,9 @@ class selinuxusermap_enable(Method):
             bool,
             doc=_(u'True means the operation was successful'),
         ),
-        output.PrimaryKey(
+        output.Output(
             'value',
+            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
@@ -553,13 +533,6 @@ class selinuxusermap_find(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
         parameters.Flag(
             'pkey_only',
             required=False,
@@ -686,13 +659,6 @@ class selinuxusermap_mod(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
     )
     has_output = (
         output.Output(
@@ -703,8 +669,9 @@ class selinuxusermap_mod(Method):
         output.Entry(
             'result',
         ),
-        output.PrimaryKey(
+        output.Output(
             'value',
+            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
@@ -736,20 +703,13 @@ class selinuxusermap_remove_host(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
         parameters.Str(
             'host',
             required=False,
             multivalue=True,
             cli_name='hosts',
             label=_(u'member host'),
-            doc=_(u'hosts to remove'),
+            doc=_(u'comma-separated list of hosts to remove'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -758,7 +718,7 @@ class selinuxusermap_remove_host(Method):
             multivalue=True,
             cli_name='hostgroups',
             label=_(u'member host group'),
-            doc=_(u'host groups to remove'),
+            doc=_(u'comma-separated list of host groups to remove'),
             alwaysask=True,
         ),
     )
@@ -805,20 +765,13 @@ class selinuxusermap_remove_user(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
         parameters.Str(
             'user',
             required=False,
             multivalue=True,
             cli_name='users',
             label=_(u'member user'),
-            doc=_(u'users to remove'),
+            doc=_(u'comma-separated list of users to remove'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -827,7 +780,7 @@ class selinuxusermap_remove_user(Method):
             multivalue=True,
             cli_name='groups',
             label=_(u'member group'),
-            doc=_(u'groups to remove'),
+            doc=_(u'comma-separated list of groups to remove'),
             alwaysask=True,
         ),
     )
@@ -881,13 +834,6 @@ class selinuxusermap_show(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
     )
     has_output = (
         output.Output(
@@ -898,8 +844,9 @@ class selinuxusermap_show(Method):
         output.Entry(
             'result',
         ),
-        output.PrimaryKey(
+        output.Output(
             'value',
+            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
