@@ -25,7 +25,7 @@ This module defines a dict, ``capabilities``, that maps feature names to API
 versions they were introduced in.
 """
 
-from distutils import version
+from ipapython.ipautil import APIVersion
 
 VERSION_WITHOUT_CAPABILITIES = u'2.51'
 
@@ -54,6 +54,10 @@ capabilities = dict(
 
     # dns_name_values: dnsnames as objects
     dns_name_values=u'2.88',
+
+    # vault supports aes key wrapping
+    vault_aes_keywrap='2.246'
+
 )
 
 
@@ -64,6 +68,6 @@ def client_has_capability(client_version, capability):
     :param client_version: The API version string reported by the client
     """
 
-    version_tuple = version.LooseVersion(client_version)
+    version = APIVersion(client_version)
 
-    return version_tuple >= version.LooseVersion(capabilities[capability])
+    return version >= APIVersion(capabilities[capability])
