@@ -41,7 +41,6 @@ import base64
 import re
 
 from cryptography import x509 as crypto_x509
-from cryptography import utils as crypto_utils
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.serialization import (
@@ -88,8 +87,7 @@ SAN_UPN = '1.3.6.1.4.1.311.20.2.3'
 SAN_KRB5PRINCIPALNAME = '1.3.6.1.5.2.2'
 
 
-@crypto_utils.register_interface(crypto_x509.Certificate)
-class IPACertificate:
+class IPACertificate(crypto_x509.Certificate):
     """
     A proxy class wrapping a python-cryptography certificate representation for
     IPA purposes
@@ -397,7 +395,15 @@ class IPACertificate:
             for value in values:
                 match_san.append(('DNS', value))
 
-        ssl.match_hostname(match_cert, DNSName(hostname).ToASCII())
+        # deprecated in Python3.7 without replacement
+        ssl.match_hostname(  # pylint: disable=deprecated-method
+            match_cert, DNSName(hostname).ToASCII()
+        )
+
+    # added in python-cryptography 38.0
+    @property
+    def tbs_precertificate_bytes(self):
+        return self._cert.tbs_precertificate_bytes
 
 
 def load_pem_x509_certificate(data):
@@ -806,12 +812,10 @@ class ExternalCAProfile:
             _oid = univ.ObjectIdentifier(parts[0])
 
             # It is; construct a V2 template
-            # pylint: disable=too-many-function-args
             return MSCSTemplateV2.__new__(MSCSTemplateV2, s)
 
         except pyasn1.error.PyAsn1Error:
             # It is not an OID; treat as a template name
-            # pylint: disable=too-many-function-args
             return MSCSTemplateV1.__new__(MSCSTemplateV1, s)
 
     def __getstate__(self):
