@@ -37,12 +37,7 @@ from ipapython import ipautil
 from ipaplatform.paths import paths
 from ipaplatform.tasks import tasks
 
-# pylint: disable=no-name-in-module, import-error
-if six.PY3:
-    from collections.abc import Mapping
-else:
-    from collections import Mapping
-# pylint: enable=no-name-in-module, import-error
+from collections.abc import Mapping
 
 logger = logging.getLogger(__name__)
 
@@ -539,4 +534,4 @@ knownservices = KnownServices({})
 
 # System may support more time&date services. FreeIPA supports chrony only.
 # Other services will be disabled during IPA installation
-timedate_services = ['ntpd', 'chronyd']
+timedate_services = ['ntpd', 'chronyd', 'systemd-timesyncd']
