@@ -27,6 +27,7 @@ from ipaplatform.paths import paths
 from ipapython import ipautil
 from ipapython.admintool import ScriptError
 import os
+import time
 
 FILES_TO_NOT_BACKUP = ['passwd', 'group', 'shadow', 'gshadow']
 
@@ -38,7 +39,7 @@ def get_auth_tool():
 
 
 @six.add_metaclass(abc.ABCMeta)
-class RedHatAuthToolBase(object):
+class RedHatAuthToolBase:
 
     @abc.abstractmethod
     def configure(self, sssd, mkhomedir, statestore, sudo=True):
@@ -100,31 +101,20 @@ class RedHatAuthSelect(RedHatAuthToolBase):
         features = output_items[1:]
         return profile, features
 
-    def configure(self, sssd, mkhomedir, statestore, sudo=True):
+    def configure(self, sssd, mkhomedir, statestore, sudo=True,
+                  subid=False):
         # In the statestore, the following keys are used for the
         # 'authselect' module:
+        # Old method:
         # profile: name of the profile configured pre-installation
         # features_list: list of features configured pre-installation
         # mkhomedir: True if installation was called with --mkhomedir
         # profile and features_list are used when reverting to the
         # pre-install state
-        cfg = self._parse_authselect_output()
-        if cfg:
-            statestore.backup_state('authselect', 'profile', cfg[0])
-            statestore.backup_state(
-                    'authselect', 'features_list', " ".join(cfg[1]))
-        else:
-            # cfg = None means that the current conf is not managed by
-            # authselect but by authconfig.
-            # As we are using authselect to configure the host,
-            # it will not be possible to revert to a custom authconfig
-            # configuration later (during uninstall)
-            # Best thing to do will be to use sssd profile at this time
-            logger.warning(
-                "WARNING: The configuration pre-client installation is not "
-                "managed by authselect and cannot be backed up. "
-                "Uninstallation may not be able to revert to the original "
-                "state.")
+        # New method:
+        # backup: name of the authselect backup
+        backup_name = "pre_ipaclient_{}".format(time.strftime("%Y%m%d%H%M%S"))
+        statestore.backup_state('authselect', 'backup', backup_name)
 
         cmd = [paths.AUTHSELECT, "select", "sssd"]
         if mkhomedir:
@@ -132,14 +122,24 @@ class RedHatAuthSelect(RedHatAuthToolBase):
             statestore.backup_state('authselect', 'mkhomedir', True)
         if sudo:
             cmd.append("with-sudo")
+        if subid:
+            cmd.append("with-subid")
         cmd.append("--force")
+        cmd.append("--backup={}".format(backup_name))
 
         ipautil.run(cmd)
 
     def unconfigure(
         self, fstore, statestore, was_sssd_installed, was_sssd_configured
     ):
-        if not statestore.has_state('authselect') and was_sssd_installed:
+        # If the installation failed before doing the authselect part
+        # nothing to do here
+        complete = statestore.get_state('installation', 'complete')
+        if complete is not None and not complete and \
+           not statestore.has_state('authselect'):
+            return
+
+        if not statestore.has_state('authselect'):
             logger.warning(
                 "WARNING: Unable to revert to the pre-installation state "
                 "('authconfig' tool has been deprecated in favor of "
@@ -167,6 +167,10 @@ class RedHatAuthSelect(RedHatAuthToolBase):
                 'authselect', 'features_list'
             )
             statestore.delete_state('authselect', 'mkhomedir')
+            # https://pagure.io/freeipa/issue/8054
+            if fstore.has_file(paths.NSSWITCH_CONF):
+                logger.info("Restoring user-nsswitch.conf")
+                fstore.restore_file(paths.NSSWITCH_CONF)
             # only non-empty features, https://pagure.io/freeipa/issue/7776
             if features_state is not None:
                 features = [
@@ -175,10 +179,15 @@ class RedHatAuthSelect(RedHatAuthToolBase):
             else:
                 features = []
 
-        cmd = [paths.AUTHSELECT, "select", profile]
-        cmd.extend(features)
-        cmd.append("--force")
-        ipautil.run(cmd)
+        backup = statestore.restore_state('authselect', 'backup')
+        if backup:
+            cmd = [paths.AUTHSELECT, "backup-restore", backup]
+            ipautil.run(cmd)
+        else:
+            cmd = [paths.AUTHSELECT, "select", profile]
+            cmd.extend(features)
+            cmd.append("--force")
+            ipautil.run(cmd)
 
     def backup(self, path):
         current = self._get_authselect_current_output()
