@@ -39,6 +39,7 @@ import time
 import textwrap
 import io
 from contextlib import contextmanager
+from configparser import RawConfigParser, ParsingError
 import locale
 import collections
 import urllib
@@ -544,7 +545,7 @@ def run(args, stdin=None, raiseonerr=True, nolog=(), env=None,
         raise
     finally:
         if skip_output:
-            p_out.close()   # pylint: disable=E1103
+            p_out.close()
 
     logger.debug('Process finished, return code=%s', p.returncode)
 
@@ -1513,7 +1514,7 @@ if six.PY2:
                 str.__name__,
                 type(value).__name__))
 else:
-    fsdecode = os.fsdecode  #pylint: disable=no-member
+    fsdecode = os.fsdecode
 
 
 def unescape_seq(seq, *args):
@@ -1757,3 +1758,41 @@ class Sleeper:
         dur = min(self.deadline - now, self.sleep)
         time.sleep(dur)
         return True
+
+
+def get_config_debug(context):
+    """A simplified version of ipalib/config in order to determine if the
+       API should be bootstrapped in debug mode or not.
+
+       A number of daemons setup logging, then bootstrap the API in order
+       to capture the startup. This presents a chicken-and-egg problem
+       over debug output.
+
+       Debug is rather spammy and probably shouldn't be enabled by default
+       but if some problem is occuring during startup we want a way to
+       see that.
+
+       So use this limited function to pluck out the debug value out of
+       the system context config file (in_tree is not supported, another
+       chicken-and-egg).
+
+       This isn't more generalized for two reasons:
+           1. There is currently only a need for 'debug'
+           2. Type conversion. They will all be strings.
+    """
+    CONFIG_SECTION = 'global'  # duplicated from ipalib.constants
+    config_file = os.path.join('/', 'etc', 'ipa', '%s.conf' % context)
+
+    parser = RawConfigParser()
+    try:
+        parser.read(config_file)
+    except ParsingError:
+        return False
+
+    if not parser.has_section(CONFIG_SECTION):
+        return False
+
+    if not parser.has_option(CONFIG_SECTION, 'debug'):
+        return False
+
+    return parser.get(CONFIG_SECTION, 'debug').lower() == 'true'
