@@ -26,6 +26,7 @@ import glob
 import errno
 import shlex
 import pipes
+import shutil
 import tempfile
 
 from augeas import Augeas
@@ -44,7 +45,7 @@ from ipapython.dn import DN
 import ipapython.errors
 from ipaserver.install import sysupgrade
 from ipalib import api, x509
-from ipalib.constants import IPAAPI_USER, MOD_SSL_VERIFY_DEPTH
+from ipalib.constants import IPAAPI_USER, MOD_SSL_VERIFY_DEPTH, IPA_CA_RECORD
 from ipaplatform.constants import constants
 from ipaplatform.tasks import tasks
 from ipaplatform.paths import paths
@@ -139,6 +140,8 @@ class HTTPInstance(service.Service):
         self.step("publish CA cert", self.__publish_ca_cert)
         self.step("clean up any existing httpd ccaches",
                   self.remove_httpd_ccaches)
+        self.step("enable ccache sweep",
+                  self.enable_ccache_sweep)
         self.step("configuring SELinux for httpd", self.configure_selinux_for_httpd)
         if not self.is_kdcproxy_configured():
             self.step("create KDC proxy config", self.create_kdcproxy_conf)
@@ -171,9 +174,15 @@ class HTTPInstance(service.Service):
         # Clean up existing ccaches
         # Make sure that empty env is passed to avoid passing KRB5CCNAME from
         # current env
-        ipautil.remove_file(paths.HTTP_CCACHE)
-        for f in os.listdir(paths.IPA_CCACHES):
-            os.remove(os.path.join(paths.IPA_CCACHES, f))
+        shutil.rmtree(paths.IPA_CCACHES)
+        ipautil.run(
+            [paths.SYSTEMD_TMPFILES, '--create', '--prefix', paths.IPA_CCACHES]
+        )
+
+    def enable_ccache_sweep(self):
+        ipautil.run(
+            [paths.SYSTEMCTL, 'enable', 'ipa-ccache-sweep.timer']
+        )
 
     def __configure_http(self):
         self.update_httpd_service_ipa_conf()
@@ -375,18 +384,29 @@ class HTTPInstance(service.Service):
             else:
                 prev_helper = None
             try:
-                certmonger.request_and_wait_for_cert(
+                # In migration case, if CA server is older version it may not
+                # have codepaths to support the ipa-ca.$DOMAIN dnsName in HTTP
+                # cert.  Therefore if request fails, try again without the
+                # ipa-ca.$DOMAIN dnsName.
+                args = dict(
                     certpath=(paths.HTTPD_CERT_FILE, paths.HTTPD_KEY_FILE),
                     principal=self.principal,
                     subject=str(DN(('CN', self.fqdn), self.subject_base)),
                     ca='IPA',
                     profile=dogtag.DEFAULT_PROFILE,
-                    dns=[self.fqdn],
+                    dns=[self.fqdn, f'{IPA_CA_RECORD}.{api.env.domain}'],
                     post_command='restart_httpd',
                     storage='FILE',
                     passwd_fname=key_passwd_file,
-                    resubmit_timeout=api.env.replication_wait_timeout
+                    resubmit_timeout=api.env.certmonger_wait_timeout,
+                    stop_tracking_on_error=True,
                 )
+                try:
+                    certmonger.request_and_wait_for_cert(**args)
+                except Exception:
+                    args['dns'] = [self.fqdn]  # remove ipa-ca.$DOMAIN
+                    args['stop_tracking_on_error'] = False
+                    certmonger.request_and_wait_for_cert(**args)
             finally:
                 if prev_helper is not None:
                     certmonger.modify_ca_helper('IPA', prev_helper)
@@ -529,7 +549,6 @@ class HTTPInstance(service.Service):
         # Remove the configuration files we create
         ipautil.remove_keytab(self.keytab)
         remove_files = [
-            paths.HTTP_CCACHE,
             paths.HTTPD_CERT_FILE,
             paths.HTTPD_KEY_FILE,
             paths.HTTPD_PASSWD_FILE_FMT.format(host=api.env.host),
@@ -593,6 +612,7 @@ class HTTPInstance(service.Service):
                 post_command='restart_httpd', storage='FILE',
                 profile=dogtag.DEFAULT_PROFILE,
                 pinfile=key_passwd_file,
+                dns=[self.fqdn, f'{IPA_CA_RECORD}.{api.env.domain}'],
             )
             subject = str(DN(cert.subject))
             certmonger.add_principal(request_id, self.principal)
