@@ -22,7 +22,6 @@ from __future__ import print_function
 
 import logging
 import os
-import pwd
 import socket
 import dbus
 
@@ -42,8 +41,6 @@ from ipapython.dn import DN
 from ipapython.dogtag import KDC_PROFILE
 
 from ipaserver.install import replication
-from ipaserver.install import ldapupdate
-
 from ipaserver.install import certs
 from ipaserver.masters import find_providing_servers
 from ipaplatform.constants import constants
@@ -54,6 +51,14 @@ logger = logging.getLogger(__name__)
 
 PKINIT_ENABLED = 'pkinitEnabled'
 
+MASTER_KEY_TYPE = 'aes256-sha2'
+SUPPORTED_ENCTYPES = ('aes256-sha2:special', 'aes128-sha2:special',
+                      'aes256-sha2:normal', 'aes128-sha2:normal',
+                      'aes256-cts:special', 'aes128-cts:special',
+                      'aes256-cts:normal', 'aes128-cts:normal',
+                      'camellia256-cts:special', 'camellia128-cts:special',
+                      'camellia256-cts:normal', 'camellia128-cts:normal')
+
 
 def get_pkinit_request_ca():
     """
@@ -162,9 +167,7 @@ class KrbInstance(service.Service):
         api.Backend.ldap2.add_entry(host_entry)
 
         # Add the host to the ipaserver host group
-        ld = ldapupdate.LDAPUpdate(ldapi=True)
-        ld.update([os.path.join(paths.UPDATES_DIR,
-                                '20-ipaservers_hostgroup.update')])
+        self._ldap_update(['20-ipaservers_hostgroup.update'])
 
     def __common_setup(self, realm_name, host_name, domain_name, admin_password):
         self.fqdn = host_name
@@ -257,6 +260,7 @@ class KrbInstance(service.Service):
         else:
             includes = ''
 
+        fips_enabled = tasks.is_fips_enabled()
         self.sub_dict = dict(FQDN=self.fqdn,
                              IP=self.ip,
                              PASSWORD=self.kdc_password,
@@ -274,7 +278,17 @@ class KrbInstance(service.Service):
                              KDC_CA_BUNDLE_PEM=paths.KDC_CA_BUNDLE_PEM,
                              CA_BUNDLE_PEM=paths.CA_BUNDLE_PEM,
                              INCLUDES=includes,
-                             FIPS='#' if tasks.is_fips_enabled() else '')
+                             FIPS='#' if fips_enabled else '')
+
+        if fips_enabled:
+            supported_enctypes = list(
+                filter(lambda e: not e.startswith('camelia'),
+                       SUPPORTED_ENCTYPES))
+        else:
+            supported_enctypes = SUPPORTED_ENCTYPES
+        self.sub_dict['SUPPORTED_ENCTYPES'] = ' '.join(supported_enctypes)
+
+        self.sub_dict['MASTER_KEY_TYPE'] = MASTER_KEY_TYPE
 
         # IPA server/KDC is not a subdomain of default domain
         # Proper domain-realm mapping needs to be specified
@@ -388,8 +402,7 @@ class KrbInstance(service.Service):
 
         self.fstore.backup_file(paths.DS_KEYTAB)
         installutils.create_keytab(paths.DS_KEYTAB, ldap_principal)
-        pent = pwd.getpwnam(constants.DS_USER)
-        os.chown(paths.DS_KEYTAB, pent.pw_uid, pent.pw_gid)
+        constants.DS_USER.chown(paths.DS_KEYTAB)
 
     def __create_host_keytab(self):
         host_principal = "host/" + self.fqdn + "@" + self.realm
@@ -455,7 +468,7 @@ class KrbInstance(service.Service):
                 subject=subject,
                 principal=krbtgt,
                 ca=certmonger_ca,
-                dns=self.fqdn,
+                dns=[self.fqdn],
                 storage='FILE',
                 profile=KDC_PROFILE,
                 post_command='renew_kdc_cert',
@@ -592,14 +605,6 @@ class KrbInstance(service.Service):
         except errors.AlreadyActive:
             pass
 
-    def __convert_to_gssapi_replication(self):
-        repl = replication.ReplicationManager(self.realm,
-                                              self.fqdn,
-                                              self.dm_password)
-        repl.convert_to_gssapi_replication(self.master_fqdn,
-                                           r_binddn=DN(('cn', 'Directory Manager')),
-                                           r_bindpw=self.dm_password)
-
     def stop_tracking_certs(self):
         certmonger.stop_tracking(certfile=paths.KDC_CERT)
 
