@@ -132,7 +132,7 @@ def read_cache(dm_password):
                                   fname,
                                   dm_password,
                                   top_dir)
-    except Exception as e:
+    except Exception:
         shutil.rmtree(top_dir)
         raise Exception("Decryption of answer cache in %s failed, please "
                         "check your password." % paths.ROOT_IPA_CACHE)
@@ -186,7 +186,7 @@ def read_host_name(host_default):
     print("Enter the fully qualified domain name of the computer")
     print("on which you're setting up server software. Using the form")
     print("<hostname>.<domainname>")
-    print("Example: master.example.com.")
+    print("Example: master.example.com")
     print("")
     print("")
     if host_default == "":
@@ -194,6 +194,9 @@ def read_host_name(host_default):
     host_name = user_input("Server host name", host_default, allow_empty=False)
     print("")
 
+    if host_name.endswith('.'):
+        host_name = host_name[:-1]
+
     return host_name
 
 
@@ -502,7 +505,7 @@ def install_check(installer):
         host_name = host_default
 
     try:
-        verify_fqdn(host_default, options.no_host_dns)
+        verify_fqdn(host_name, options.no_host_dns)
     except BadHostError as e:
         raise ScriptError(e)
 
@@ -991,6 +994,8 @@ def install(installer):
             args.append("--no-sshd")
         if options.mkhomedir:
             args.append("--mkhomedir")
+        if options.subid:
+            args.append("--subid")
         start = time.time()
         run(args, redirect_output=True)
         dur = time.time() - start
