@@ -733,7 +733,7 @@ class LDAPObject(Object):
         container_dns = {}
         new_attrs = {}
 
-        for attr in self.attribute_members:
+        for attr, members in self.attribute_members.items():
             try:
                 value = entry_attrs.raw[attr]
             except KeyError:
@@ -742,7 +742,7 @@ class LDAPObject(Object):
 
             for member in value:
                 memberdn = DN(member.decode('utf-8'))
-                for ldap_obj_name in self.attribute_members[attr]:
+                for ldap_obj_name in members:
                     ldap_obj = self.api.Object[ldap_obj_name]
                     try:
                         container_dn = container_dns[ldap_obj_name]
@@ -1135,7 +1135,11 @@ last, after all sets and adds."""),
 
                 for delval in deldict.get(attr, []):
                     try:
-                        entry_attrs[attr].remove(delval)
+                        try:
+                            val = ldap.decode(delval.encode('utf-8'), attr)
+                            entry_attrs[attr].remove(val)
+                        except ValueError:
+                            entry_attrs[attr].remove(delval)
                     except ValueError:
                         if isinstance(delval, bytes):
                             # This is a Binary value, base64 encode it
@@ -1788,7 +1792,7 @@ class LDAPAddMember(LDAPModMember):
         completed = 0
         for (attr, objs) in member_dns.items():
             for ldap_obj_name in objs:
-                for m_dn in member_dns[attr][ldap_obj_name]:
+                for m_dn in objs[ldap_obj_name]:
                     assert isinstance(m_dn, DN)
                     if not m_dn:
                         continue
