@@ -191,6 +191,7 @@ class i18n_messages(Command):
             "type_radius": _("RADIUS"),
             "type_pkinit": _("PKINIT"),
             "type_hardened": _("Hardened Password (by SPAKE or FAST)"),
+            "type_idp": _("External Identity Provider"),
             "type_disabled": _("Disable per-user override"),
             "user_tooltip": _("<p>Per-user setting, overwrites the global setting if any option is checked.</p><p><strong>Password + Two-factor:</strong> LDAP and Kerberos allow authentication with either one of the authentication types but Kerberos uses pre-authentication method which requires to use armor ccache.</p><p><strong>RADIUS with another type:</strong> Kerberos always use RADIUS, but LDAP never does. LDAP only recognize the password and two-factor authentication options.</p>"),
         },
@@ -1091,6 +1092,16 @@ class i18n_messages(Command):
                     "Remove host groups from host group '${primary_key}'"
                 ),
             },
+            "idp": {
+                "template_keycloak": _("Keycloak or Red Hat SSO"),
+                "template_google": _("Google"),
+                "template_github": _("Github"),
+                "template_microsoft": _("Microsoft or Azure"),
+                "template_okta": _("Okta"),
+                "label_idpclient": _("OAuth 2.0 client details"),
+                "label_idp": _("Identity provider details"),
+                "verify_secret": _("Verify secret"),
+            },
             "idoverrideuser": {
                 "anchor_label": _("User to override"),
                 "anchor_tooltip": _("Enter trusted or IPA user login. Note: search doesn't list users from trusted domains."),
