@@ -73,7 +73,7 @@ try:
 except Exception as e:
     _nss_idmap_installed = False
 
-if api.env.in_server and api.env.context in ['lite', 'server']:
+if api.env.in_server:
     try:
         import ipaserver.dcerpc
         _bindings_installed = True
@@ -546,11 +546,11 @@ class trust(LDAPObject):
             flags=['no_create', 'no_update']),
         Str('ipantsidblacklistincoming*',
             cli_name='sid_blacklist_incoming',
-            label=_('SID blacklist incoming'),
+            label=_('SID blocklist incoming'),
             flags=['no_create']),
         Str('ipantsidblacklistoutgoing*',
             cli_name='sid_blacklist_outgoing',
-            label=_('SID blacklist outgoing'),
+            label=_('SID blocklist outgoing'),
             flags=['no_create']),
         Str('trustdirection',
             label=_('Trust direction'),
@@ -571,7 +571,7 @@ class trust(LDAPObject):
         ),
     )
 
-    def validate_sid_blacklists(self, entry_attrs):
+    def validate_sid_blocklists(self, entry_attrs):
         if not _bindings_installed:
             # SID validator is not available, return
             # Even if invalid SID gets in the trust entry, it won't crash
@@ -614,6 +614,7 @@ class trust(LDAPObject):
                     ldap.SCOPE_SUBTREE, trustfilter, ['']
                 )
             except errors.NotFound:
+                # pylint: disable=raising-bad-type, #4772
                 raise self.handle_not_found(keys[-1])
 
             if len(result) > 1:
@@ -789,7 +790,7 @@ ipa idrange-del before retrying the command with the desired range type.
         if (options.get('trust_type') == u'ad' and
                 options.get('trust_secret') is None):
 
-            if options.get('bidirectional') == True:
+            if options.get('bidirectional'):
                 # Bidirectional trust allows us to use cross-realm TGT,
                 # so we can run the call under original user's credentials
                 res = fetch_domains_from_trust(self.api, self.trustinstance,
@@ -1130,7 +1131,7 @@ class trust_mod(LDAPUpdate):
     def pre_callback(self, ldap, dn, e_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
 
-        self.obj.validate_sid_blacklists(e_attrs)
+        self.obj.validate_sid_blocklists(e_attrs)
 
         return dn
 
@@ -1619,13 +1620,13 @@ class trustdomain_find(LDAPSearch):
             return truncated
         trust_dn = self.obj.get_dn(args[0], trust_type=u'ad')
         trust_entry = ldap.get_entry(trust_dn)
-        blacklist = trust_entry.get('ipantsidblacklistincoming')
+        blocklist = trust_entry.get('ipantsidblacklistincoming')
         for entry in entries:
             sid = entry.get('ipanttrusteddomainsid', [None])[0]
             if sid is None:
                 continue
 
-            if sid in blacklist:
+            if sid in blocklist:
                 entry['domain_enabled'] = [False]
             else:
                 entry['domain_enabled'] = [True]
@@ -1744,6 +1745,9 @@ def add_new_domains_from_trust(myapi, trustinstance, trust_entry,
         tlns = entry.get('ipantadditionalsuffixes', [])
         tlns.extend(x for x in suffixes if x not in tlns)
         entry['ipantadditionalsuffixes'] = tlns
+        ftidata = domains.get('ftinfo_data', None)
+        if ftidata is not None:
+            entry['ipanttrustforesttrustinfo'] = [ftidata]
         ldap.update_entry(entry)
     except errors.EmptyModlist:
         pass
