@@ -28,8 +28,10 @@ from __future__ import absolute_import
 import logging
 from xml.sax.saxutils import escape
 import os
+import time
 import traceback
 from io import BytesIO
+from sys import version_info
 from urllib.parse import parse_qs
 from xmlrpc.client import Fault
 
@@ -46,9 +48,11 @@ from ipalib.capabilities import VERSION_WITHOUT_CAPABILITIES
 from ipalib.frontend import Local
 from ipalib.install.kinit import kinit_armor, kinit_password
 from ipalib.backend import Executioner
-from ipalib.errors import (PublicError, InternalError, JSONError,
+from ipalib.errors import (
+    PublicError, InternalError, JSONError,
     CCacheError, RefererError, InvalidSessionPassword, NotFound, ACIError,
-    ExecutionError, PasswordExpired, KrbPrincipalExpired, UserLocked)
+    ExecutionError, PasswordExpired, KrbPrincipalExpired, KrbPrincipalWrongFAST,
+    UserLocked)
 from ipalib.request import context, destroy_context
 from ipalib.rpc import (xml_dumps, xml_loads,
     json_encode_binary, json_decode_binary)
@@ -69,10 +73,15 @@ from requests.auth import AuthBase
 if six.PY3:
     unicode = str
 
+# time.perf_counter_ns appeared in Python 3.7.
+if version_info < (3, 7):
+    time.perf_counter_ns = lambda: int(time.perf_counter() * 10**9)
+
 logger = logging.getLogger(__name__)
 
 HTTP_STATUS_SUCCESS = '200 Success'
 HTTP_STATUS_SERVER_ERROR = '500 Internal Server Error'
+HTTP_STATUS_SERVICE_UNAVAILABLE = "503 Service Unavailable"
 
 _not_found_template = """<html>
 <head>
@@ -122,6 +131,18 @@ _unauthorized_template = """<html>
 </body>
 </html>"""
 
+_service_unavailable_template = """<html>
+<head>
+<title>503 Service Unavailable</title>
+</head>
+<body>
+<h1>Service Unavailable</h1>
+<p>
+<strong>%(message)s</strong>
+</p>
+</body>
+</html>"""
+
 _success_template = """<html>
 <head>
 <title>200 Success</title>
@@ -188,6 +209,20 @@ class HTTP_Status(plugable.Plugin):
         output = _unauthorized_template % dict(message=escape(message))
         return [output.encode('utf-8')]
 
+    def service_unavailable(self, environ, start_response, message):
+        """
+        Return a 503 Service Unavailable
+        """
+        status = HTTP_STATUS_SERVICE_UNAVAILABLE
+        response_headers = [('Content-Type', 'text/html; charset=utf-8')]
+
+        logger.error('%s: %s', status, message)
+
+        start_response(status, response_headers)
+        output = _service_unavailable_template % dict(message=escape(message))
+        return [output.encode('utf-8')]
+
+
 def read_input(environ):
     """
     Read the request body from environ['wsgi.input'].
@@ -341,11 +376,15 @@ class WSGIExecutioner(Executioner):
         command = None
 
         e = None
-        if not 'HTTP_REFERER' in environ:
+        if 'HTTP_REFERER' not in environ:
             return self.marshal(result, RefererError(referer='missing'), _id)
         if not environ['HTTP_REFERER'].startswith('https://%s/ipa' % self.api.env.host) and not self.env.in_tree:
             return self.marshal(result, RefererError(referer=environ['HTTP_REFERER']), _id)
+        if self.api.env.debug:
+            time_start = time.perf_counter_ns()
         try:
+            if 'KRB5CCNAME' in environ:
+                setattr(context, "ccache_name", environ['KRB5CCNAME'])
             if ('HTTP_ACCEPT_LANGUAGE' in environ):
                 lang_reg_w_q = environ['HTTP_ACCEPT_LANGUAGE'].split(',')[0]
                 lang_reg = lang_reg_w_q.split(';')[0]
@@ -385,6 +424,8 @@ class WSGIExecutioner(Executioner):
             try:
                 params = command.args_options_2_params(*args, **options)
             except Exception as e:
+                if self.api.env.debug:
+                    time_end = time.perf_counter_ns()
                 logger.info(
                    'exception %s caught when converting options: %s',
                    e.__class__.__name__, str(e)
@@ -392,6 +433,9 @@ class WSGIExecutioner(Executioner):
                 # get at least some context of what is going on
                 params = options
                 error = e
+            else:
+                if self.api.env.debug:
+                    time_end = time.perf_counter_ns()
             if error:
                 result_string = type(error).__name__
             else:
@@ -402,6 +446,14 @@ class WSGIExecutioner(Executioner):
                         name,
                         ', '.join(command._repr_iter(**params)),
                         result_string)
+            if self.api.env.debug:
+                logger.debug('[%s] %s: %s(%s): %s %s',
+                             type(self).__name__,
+                             principal,
+                             name,
+                             ', '.join(command._repr_iter(**params)),
+                             result_string,
+                             'etime=' + str(time_end - time_start))
         else:
             logger.info('[%s] %s: %s: %s',
                         type(self).__name__,
@@ -694,7 +746,7 @@ class KerberosWSGIExecutioner(WSGIExecutioner, KerberosSession):
             status = HTTP_STATUS_SUCCESS
             response = status.encode('utf-8')
             start_response(status, self.headers)
-            return self.marshal(None, e)
+            return [self.marshal(None, e)]
         finally:
             destroy_context()
         return response
@@ -858,6 +910,18 @@ class jsonserver_session(jsonserver, KerberosSession):
             self.create_context(ccache=ccache_name)
         except ACIError as e:
             return self.unauthorized(environ, start_response, str(e), 'denied')
+        except errors.DatabaseError as e:
+            # account is disable but user has a valid ticket
+            msg = str(e)
+            if "account inactivated" in msg.lower():
+                return self.unauthorized(
+                    environ, start_response, str(e), "account disabled"
+                )
+            else:
+                return self.service_unavailable(environ, start_response, msg)
+
+        except CCacheError:
+            return self.need_login(start_response)
 
         try:
             response = super(jsonserver_session, self).__call__(environ, start_response)
@@ -921,6 +985,34 @@ class login_password(Backend, KerberosSession):
         self.api.Backend.wsgi_dispatch.mount(self, self.key)
 
     def __call__(self, environ, start_response):
+        def attempt_kinit(user_principal, password,
+                          ipa_ccache_name, use_armor=True):
+            try:
+                # try to remove in case an old file was there
+                os.unlink(ipa_ccache_name)
+            except OSError:
+                pass
+            try:
+                self.kinit(user_principal, password,
+                           ipa_ccache_name, use_armor=use_armor)
+            except PasswordExpired as e:
+                return self.unauthorized(environ, start_response,
+                                         str(e), 'password-expired')
+            except InvalidSessionPassword as e:
+                return self.unauthorized(environ, start_response,
+                                         str(e), 'invalid-password')
+            except KrbPrincipalExpired as e:
+                return self.unauthorized(environ,
+                                         start_response,
+                                         str(e),
+                                         'krbprincipal-expired')
+            except UserLocked as e:
+                return self.unauthorized(environ,
+                                         start_response,
+                                         str(e),
+                                         'user-locked')
+            return None
+
         logger.debug('WSGI login_password.__call__:')
 
         # Get the user and password parameters from the request
@@ -936,7 +1028,7 @@ class login_password(Backend, KerberosSession):
 
         try:
             query_dict = parse_qs(query_string)
-        except Exception as e:
+        except Exception:
             return self.bad_request(environ, start_response, "cannot parse query data")
 
         user = query_dict.get('user', None)
@@ -971,26 +1063,14 @@ class login_password(Backend, KerberosSession):
         ipa_ccache_name = os.path.join(paths.IPA_CCACHES,
                                        'kinit_{}'.format(os.getpid()))
         try:
-            # try to remove in case an old file was there
-            os.unlink(ipa_ccache_name)
-        except OSError:
-            pass
-        try:
-            self.kinit(user_principal, password, ipa_ccache_name)
-        except PasswordExpired as e:
-            return self.unauthorized(environ, start_response, str(e), 'password-expired')
-        except InvalidSessionPassword as e:
-            return self.unauthorized(environ, start_response, str(e), 'invalid-password')
-        except KrbPrincipalExpired as e:
-            return self.unauthorized(environ,
-                                     start_response,
-                                     str(e),
-                                     'krbprincipal-expired')
-        except UserLocked as e:
-            return self.unauthorized(environ,
-                                     start_response,
-                                     str(e),
-                                     'user-locked')
+            result = attempt_kinit(user_principal, password,
+                                   ipa_ccache_name, use_armor=True)
+        except KrbPrincipalWrongFAST:
+            result = attempt_kinit(user_principal, password,
+                                   ipa_ccache_name, use_armor=False)
+
+        if result is not None:
+            return result
 
         result = self.finalize_kerberos_acquisition('login_password',
                                                     ipa_ccache_name, environ,
@@ -1002,21 +1082,24 @@ class login_password(Backend, KerberosSession):
             pass
         return result
 
-    def kinit(self, principal, password, ccache_name):
-        # get anonymous ccache as an armor for FAST to enable OTP auth
-        armor_path = os.path.join(paths.IPA_CCACHES,
-                                  "armor_{}".format(os.getpid()))
+    def kinit(self, principal, password, ccache_name, use_armor=True):
+        if use_armor:
+            # get anonymous ccache as an armor for FAST to enable OTP auth
+            armor_path = os.path.join(paths.IPA_CCACHES,
+                                      "armor_{}".format(os.getpid()))
 
-        logger.debug('Obtaining armor in ccache %s', armor_path)
+            logger.debug('Obtaining armor in ccache %s', armor_path)
 
-        try:
-            kinit_armor(
-                armor_path,
-                pkinit_anchors=[paths.KDC_CERT, paths.KDC_CA_BUNDLE_PEM],
-            )
-        except RuntimeError as e:
-            logger.error("Failed to obtain armor cache")
-            # We try to continue w/o armor, 2FA will be impacted
+            try:
+                kinit_armor(
+                    armor_path,
+                    pkinit_anchors=[paths.KDC_CERT, paths.KDC_CA_BUNDLE_PEM],
+                )
+            except RuntimeError:
+                logger.error("Failed to obtain armor cache")
+                # We try to continue w/o armor, 2FA will be impacted
+                armor_path = None
+        else:
             armor_path = None
 
         try:
@@ -1026,6 +1109,7 @@ class login_password(Backend, KerberosSession):
                 ccache_name,
                 armor_ccache_name=armor_path,
                 enterprise=True,
+                canonicalize=True,
                 lifetime=self.api.env.kinit_lifetime)
 
             if armor_path:
@@ -1044,6 +1128,9 @@ class login_password(Backend, KerberosSession):
                   'while getting initial credentials') in str(e):
                 raise UserLocked(principal=principal,
                                  message=unicode(e))
+            elif ('kinit: Error constructing AP-REQ armor: '
+                  'Matching credential not found') in str(e):
+                raise KrbPrincipalWrongFAST(principal=principal)
             raise InvalidSessionPassword(principal=principal,
                                          message=unicode(e))
 
@@ -1073,8 +1160,10 @@ class change_password(Backend, HTTP_Status):
 
         try:
             query_dict = parse_qs(query_string)
-        except Exception as e:
-            return self.bad_request(environ, start_response, "cannot parse query data")
+        except Exception:
+            return self.bad_request(
+                environ, start_response, "cannot parse query data"
+            )
 
         data = {}
         for field in ('user', 'old_password', 'new_password', 'otp'):
@@ -1178,8 +1267,10 @@ class sync_token(Backend, HTTP_Status):
         # Parse the query string to a dictionary.
         try:
             query_dict = parse_qs(query_string)
-        except Exception as e:
-            return self.bad_request(environ, start_response, "cannot parse query data")
+        except Exception:
+            return self.bad_request(
+                environ, start_response, "cannot parse query data"
+            )
         data = {}
         for field in ('user', 'password', 'first_code', 'second_code', 'token'):
             value = query_dict.get(field, None)
