@@ -100,7 +100,9 @@ class TestClientInstallation(IntegrationTest):
             ['rm', '-f', '/etc/authselect/authselect.conf'])
         result = self._install_client()
         assert result.returncode == 0
-        assert self.msg_warn_install in result.stderr_text
+        # With the fix for 8189, there is no warning any more
+        # because install is performing a pre-install backup
+        assert self.msg_warn_install not in result.stderr_text
         # Client installation must configure the 'sssd' profile
         # with sudo
         check_authselect_profile(self.client, default_profile, ('with-sudo',))
@@ -109,12 +111,13 @@ class TestClientInstallation(IntegrationTest):
         """
         Test client un-installation when there was no authselect profile
         """
-        # As the client did not have any authselect profile before install,
-        # uninstall must print a warning about restoring 'sssd' profile
-        # by default
+        # The client did not have any authselect profile before install,
+        # but uninstall must be able to restore the backup
+        # Check that no profile is configured after uninstall
         result = self._uninstall_client()
         assert result.returncode == 0
-        check_authselect_profile(self.client, default_profile)
+        assert not self.client.transport.file_exists(
+            '/etc/authselect/authselect.conf')
 
     def test_install_client_preconfigured_profile(self):
         """
@@ -189,6 +192,17 @@ class TestClientInstallation(IntegrationTest):
         result = self._uninstall_client()
         assert result.returncode == 0
 
+    def test_install_client_subid(self):
+        """
+        Test client installation with --subid option
+        """
+        result = self._install_client(extraargs=['-f', '--subid'])
+        assert result.returncode == 0
+        # Client installation must configure the 'sssd' profile
+        # with subid feature
+        check_authselect_profile(
+            self.client, default_profile, ('with-sudo', 'with-subid'))
+
     @classmethod
     def uninstall(cls, mh):
         super(TestClientInstallation, cls).uninstall(mh)
@@ -237,6 +251,23 @@ class TestServerInstallation(IntegrationTest):
         check_authselect_profile(
             self.master, preconfigured_profile, preconfigured_options)
 
+    def test_install_with_subid(self):
+        """
+        Test server installation when --subid option is specified
+        """
+        tasks.install_master(self.master, extra_args=["--subid"])
+        check_authselect_profile(
+            self.master, default_profile, ('with-sudo', 'with-subid'))
+
+    def test_uninstall_with_subid(self):
+        """
+        Test server uninstallation when --subid option was configured
+        """
+        # uninstall must revert to the preconfigured profile
+        tasks.uninstall_master(self.master)
+        check_authselect_profile(
+            self.master, preconfigured_profile, preconfigured_options)
+
     @classmethod
     def uninstall(cls, mh):
         # Clean-up the authselect profile and re-use the default 'sssd'
