@@ -35,11 +35,13 @@ from ipatests.test_integration.base import IntegrationTest
 from ipatests.pytest_ipa.integration import tasks
 from ipatests.test_integration.test_dnssec import wait_until_record_is_signed
 from ipatests.test_integration.test_simple_replication import check_replication
+from ipatests.test_integration.test_topology import find_segment
 from ipatests.util import assert_deepequal
 from ldap.dn import escape_dn_chars
 
 logger = logging.getLogger(__name__)
 
+
 def assert_entries_equal(a, b):
     assert_deepequal(a.dn, b.dn)
     assert_deepequal(dict(a), dict(b))
@@ -161,28 +163,7 @@ def restore_checker(host):
         assert_func(expected, got)
 
 
-def backup(host):
-    """Run backup on host, return the path to the backup directory"""
-    result = host.run_command(['ipa-backup', '-v'])
-
-    # Test for ticket 7632: check that services are restarted
-    # before the backup is compressed
-    pattern = r'.*{}.*Starting IPA service.*'.format(paths.GZIP)
-    if (re.match(pattern, result.stderr_text, re.DOTALL)):
-        raise AssertionError('IPA Services are started after compression')
-
-    # Get the backup location from the command's output
-    for line in result.stderr_text.splitlines():
-        prefix = 'ipaserver.install.ipa_backup: INFO: Backed up to '
-        if line.startswith(prefix):
-            backup_path = line[len(prefix):].strip()
-            logger.info('Backup path for %s is %s', host.hostname, backup_path)
-            return backup_path
-    else:
-        raise AssertionError('Backup directory not found in output')
-
-
-@pytest.yield_fixture(scope="function")
+@pytest.fixture
 def cert_sign_request(request):
     master = request.instance.master
     hosts = [master] + request.instance.replicas
@@ -206,7 +187,7 @@ class TestBackupAndRestore(IntegrationTest):
     def test_full_backup_and_restore(self):
         """backup, uninstall, restore"""
         with restore_checker(self.master):
-            backup_path = backup(self.master)
+            backup_path = tasks.get_backup_dir(self.master)
 
             self.master.run_command(['ipa-server-install',
                                      '--uninstall',
@@ -232,7 +213,7 @@ class TestBackupAndRestore(IntegrationTest):
     def test_full_backup_and_restore_with_removed_users(self):
         """regression test for https://fedorahosted.org/freeipa/ticket/3866"""
         with restore_checker(self.master):
-            backup_path = backup(self.master)
+            backup_path = tasks.get_backup_dir(self.master)
 
             logger.info('Backup path for %s is %s', self.master, backup_path)
 
@@ -258,7 +239,7 @@ class TestBackupAndRestore(IntegrationTest):
     def test_full_backup_and_restore_with_selinux_booleans_off(self):
         """regression test for https://fedorahosted.org/freeipa/ticket/4157"""
         with restore_checker(self.master):
-            backup_path = backup(self.master)
+            backup_path = tasks.get_backup_dir(self.master)
 
             logger.info('Backup path for %s is %s', self.master, backup_path)
 
@@ -311,7 +292,7 @@ class BaseBackupAndRestoreWithDNS(IntegrationTest):
 
             tasks.resolve_record(self.master.ip, self.example_test_zone)
 
-            backup_path = backup(self.master)
+            backup_path = tasks.get_backup_dir(self.master)
 
             self.master.run_command(['ipa-server-install',
                                      '--uninstall',
@@ -331,6 +312,11 @@ class BaseBackupAndRestoreWithDNS(IntegrationTest):
                 tasks.install_master(self.master, setup_dns=True)
             self.master.run_command(['ipa-restore', backup_path],
                                     stdin_text=dirman_password + '\nyes')
+            if reinstall:
+                # If the server was reinstalled, reinstall may have changed
+                # the uid and restore reverts to the original value.
+                # clear the cache to make sure we get up-to-date values
+                tasks.clear_sssd_cache(self.master)
             tasks.resolve_record(self.master.ip, self.example_test_zone)
 
             tasks.kinit_admin(self.master)
@@ -387,7 +373,7 @@ class BaseBackupAndRestoreWithDNSSEC(IntegrationTest):
                     self.master.ip, self.example_test_zone)
             ), "Zone is not signed"
 
-            backup_path = backup(self.master)
+            backup_path = tasks.get_backup_dir(self.master)
 
             self.master.run_command(['ipa-server-install',
                                      '--uninstall',
@@ -400,6 +386,12 @@ class BaseBackupAndRestoreWithDNSSEC(IntegrationTest):
             self.master.run_command(['ipa-restore', backup_path],
                                     stdin_text=dirman_password + '\nyes')
 
+            if reinstall:
+                # If the server was reinstalled, reinstall may have changed
+                # the uid and restore reverts to the original value.
+                # clear the cache to make sure we get up-to-date values
+                tasks.clear_sssd_cache(self.master)
+
             assert (
                 wait_until_record_is_signed(
                     self.master.ip, self.example_test_zone)
@@ -469,11 +461,13 @@ class BaseBackupAndRestoreWithKRA(IntegrationTest):
                 "--password", self.vault_password,
             ])
 
-            backup_path = backup(self.master)
+            backup_path = tasks.get_backup_dir(self.master)
 
-            self.master.run_command(['ipa-server-install',
-                                     '--uninstall',
-                                     '-U'])
+            # check that no error message in uninstall log for KRA instance
+            cmd = self.master.run_command(['ipa-server-install',
+                                           '--uninstall',
+                                           '-U'])
+            assert "failed to uninstall KRA" not in cmd.stderr_text
 
             if reinstall:
                 tasks.install_master(self.master, setup_dns=True)
@@ -482,6 +476,12 @@ class BaseBackupAndRestoreWithKRA(IntegrationTest):
             self.master.run_command(['ipa-restore', backup_path],
                                     stdin_text=dirman_password + '\nyes')
 
+            if reinstall:
+                # If the server was reinstalled, reinstall may have changed
+                # the uid and restore reverts to the original value.
+                # clear the cache to make sure we get up-to-date values
+                tasks.clear_sssd_cache(self.master)
+
             tasks.kinit_admin(self.master)
             # retrieve secret after restore
             self.master.run_command([
@@ -502,6 +502,20 @@ class TestBackupReinstallRestoreWithKRA(BaseBackupAndRestoreWithKRA):
         """backup, uninstall, reinstall, restore"""
         self._full_backup_restore_with_vault(reinstall=True)
 
+    def test_no_error_message_with_uninstall_ipa_with_kra(self):
+        """Test there is no error message in uninstall log for KRA instance
+
+        There was error message in uninstall log when IPA with KRA was
+        uninstalled. This test check that there is no error message in
+        uninstall log for kra instance.
+
+        related: https://pagure.io/freeipa/issue/8550
+        """
+        cmd = self.master.run_command(['ipa-server-install',
+                                       '--uninstall',
+                                       '-U'])
+        assert "failed to uninstall KRA" not in cmd.stderr_text
+
 
 class TestBackupAndRestoreWithReplica(IntegrationTest):
     """Regression tests for issues 7234 and 7455
@@ -526,15 +540,6 @@ class TestBackupAndRestoreWithReplica(IntegrationTest):
             domain_level = cls.master.config.domain_level
         else:
             domain_level = cls.domain_level
-        # Configure /etc/resolv.conf on each replica to use the master as DNS
-        # Otherwise ipa-replica-manage re-initialize is unable to
-        # resolve the master name
-        tasks.config_host_resolvconf_with_master_data(
-            cls.master, cls.replica1
-        )
-        tasks.config_host_resolvconf_with_master_data(
-            cls.master, cls.replica2
-        )
         # Configure only master and one replica.
         # Replica is configured without CA
         tasks.install_topo(
@@ -588,7 +593,7 @@ class TestBackupAndRestoreWithReplica(IntegrationTest):
         tasks.user_add(self.replica1, 'test1_replica')
 
         with restore_checker(self.master):
-            backup_path = backup(self.master)
+            backup_path = tasks.get_backup_dir(self.master)
 
             # change data after backup
             self.master.run_command(['ipa', 'user-del', 'test1_master'])
@@ -712,7 +717,7 @@ class TestUserRootFilesOwnershipPermission(IntegrationTest):
     def test_userroot_ldif_files_ownership_and_permission(self):
         """backup, uninstall, restore, backup"""
         tasks.install_master(self.master)
-        backup_path = backup(self.master)
+        backup_path = tasks.get_backup_dir(self.master)
 
         self.master.run_command(['ipa-server-install',
                                  '--uninstall',
@@ -778,7 +783,7 @@ class TestBackupAndRestoreDMPassword(IntegrationTest):
     def test_restore_bad_dm_password(self):
         """backup, uninstall, restore, wrong DM password (expect failure)"""
         with restore_checker(self.master):
-            backup_path = backup(self.master)
+            backup_path = tasks.get_backup_dir(self.master)
 
             # No uninstall, just pure restore, the only case where
             # prompting for the DM password matters.
@@ -792,7 +797,7 @@ class TestBackupAndRestoreDMPassword(IntegrationTest):
 
         # Flying blind without the restore_checker so we can have
         # an error thrown when dirsrv is down.
-        backup_path = backup(self.master)
+        backup_path = tasks.get_backup_dir(self.master)
 
         self.master.run_command(['ipactl', 'stop'])
 
@@ -827,7 +832,7 @@ class TestReplicaInstallAfterRestore(IntegrationTest):
         check_replication(master, replica1, "testuser1")
 
         # backup master.
-        backup_path = backup(master)
+        backup_path = tasks.get_backup_dir(self.master)
 
         suffix = ipautil.realm_to_suffix(master.domain.realm)
         suffix = escape_dn_chars(str(suffix))
@@ -858,13 +863,14 @@ class TestReplicaInstallAfterRestore(IntegrationTest):
                            stdin_text=dirman_password + '\nyes')
 
         # re-initialize topology after restore.
-        topo_name = "{}-to-{}".format(master.hostname, replica1.hostname)
         for topo_suffix in 'domain', 'ca':
-            arg = ['ipa',
-                   'topologysegment-reinitialize',
-                   topo_suffix,
-                   topo_name,
-                   '--left']
+            topo_name = find_segment(master, replica1, topo_suffix)
+            arg = ['ipa', 'topologysegment-reinitialize',
+                   topo_suffix, topo_name]
+            if topo_name.split('-to-', maxsplit=1)[0] != master.hostname:
+                arg.append('--left')
+            else:
+                arg.append('--right')
             replica1.run_command(arg)
 
         # wait sometime for re-initialization
@@ -891,7 +897,7 @@ class TestBackupAndRestoreTrust(IntegrationTest):
                                  '--add-sids'])
 
         with restore_checker(self.master):
-            backup_path = backup(self.master)
+            backup_path = tasks.get_backup_dir(self.master)
 
             self.master.run_command(['ipa-server-install',
                                      '--uninstall',
@@ -909,3 +915,222 @@ class TestBackupAndRestoreTrust(IntegrationTest):
             tasks.install_packages(self.master, ['*ipa-server-trust-ad'])
             self.master.run_command(['ipa-restore', backup_path],
                                     stdin_text=dirman_password + '\nyes')
+
+
+class TestBackupRoles(IntegrationTest):
+    """ipa-backup should only run on replicas with all the in-use roles
+    https://pagure.io/freeipa/issue/8217
+    """
+
+    num_replicas = 1
+    topology = 'star'
+
+    serverroles = {
+        'ADTA': u'AD trust agent',
+        'ADTC': u'AD trust controller',
+        'CA': u'CA server',
+        'DNS': u'DNS server',
+        'KRA': u'KRA server'
+    }
+
+    @classmethod
+    def install(cls, mh):
+        tasks.install_master(cls.master, setup_dns=True)
+
+    def _check_rolecheck_backup_failure(self, host):
+        result = tasks.ipa_backup(host, raiseonerr=False)
+        assert result.returncode == 1
+        assert "Error: Local roles" in result.stderr_text
+        assert "do not match globally used roles" in result.stderr_text
+
+        result = tasks.ipa_backup(host, disable_role_check=True)
+        assert result.returncode == 0
+        assert "Warning: Local roles" in result.stderr_text
+        assert "do not match globally used roles" in result.stderr_text
+
+    def _check_rolecheck_backup_success(self, host):
+        result = tasks.ipa_backup(host)
+        assert result.returncode == 0
+        assert "Local roles match globally used roles, proceeding." \
+            in result.stderr_text
+
+    def _ipa_replica_role_check(self, replica, role):
+        return "enabled" in self.master.run_command([
+            'ipa', 'server-role-find',
+            '--server', replica,
+            '--role', role
+        ]).stdout_text
+
+    def test_rolecheck_DNS_CA(self):
+        """ipa-backup rolecheck:
+        start with a master with DNS and CA then
+        gradually upgrade a replica to the DNS and CA
+        roles.
+        """
+
+        # single master: check that backup works.
+        assert self._ipa_replica_role_check(
+            self.master.hostname, self.serverroles['DNS']
+        )
+        assert self._ipa_replica_role_check(
+            self.master.hostname, self.serverroles['CA']
+        )
+        assert not self._ipa_replica_role_check(
+            self.master.hostname, self.serverroles['KRA']
+        )
+        self._check_rolecheck_backup_success(self.master)
+
+        # install CA-less, DNS-less replica
+        tasks.install_replica(self.master, self.replicas[0], setup_ca=False)
+        assert not self._ipa_replica_role_check(
+            self.replicas[0].hostname, self.serverroles['DNS']
+        )
+        assert not self._ipa_replica_role_check(
+            self.replicas[0].hostname, self.serverroles['CA']
+        )
+        assert not self._ipa_replica_role_check(
+            self.replicas[0].hostname, self.serverroles['KRA']
+        )
+        self._check_rolecheck_backup_success(self.master)
+        self._check_rolecheck_backup_failure(self.replicas[0])
+
+        # install DNS on replica
+        tasks.install_dns(self.replicas[0])
+        assert self._ipa_replica_role_check(
+            self.replicas[0].hostname, self.serverroles['DNS']
+        )
+        self._check_rolecheck_backup_failure(self.replicas[0])
+
+    def test_rolecheck_KRA(self):
+        """ipa-backup rolecheck: add a KRA.
+        """
+
+        # install CA on replica.
+        tasks.install_ca(self.replicas[0])
+        # master and replicas[0] have matching roles now.
+        assert self._ipa_replica_role_check(
+            self.replicas[0].hostname, self.serverroles['CA']
+        )
+        self._check_rolecheck_backup_success(self.master)
+        self._check_rolecheck_backup_success(self.replicas[0])
+
+        # install KRA on replica
+        tasks.install_kra(self.replicas[0], first_instance=True)
+        assert self._ipa_replica_role_check(
+            self.replicas[0].hostname, self.serverroles['KRA']
+        )
+        self._check_rolecheck_backup_success(self.replicas[0])
+        self._check_rolecheck_backup_failure(self.master)
+
+        # install KRA on master
+        tasks.install_kra(self.master)
+        # master and replicas[0] have matching roles now.
+        assert self._ipa_replica_role_check(
+            self.master.hostname, self.serverroles['KRA']
+        )
+        self._check_rolecheck_backup_success(self.replicas[0])
+        self._check_rolecheck_backup_success(self.master)
+
+    def test_rolecheck_Trust(self):
+        """ipa-backup rolecheck: add Trust controllers/agents.
+        """
+
+        # install AD Trust packages on replica first
+        tasks.install_packages(self.replicas[0], ['*ipa-server-trust-ad'])
+        self._check_rolecheck_backup_success(self.replicas[0])
+        self._check_rolecheck_backup_success(self.master)
+        tasks.install_packages(self.master, ['*ipa-server-trust-ad'])
+
+        # make replicas[0] become Trust Controller
+        self.replicas[0].run_command([
+            'ipa-adtrust-install', '-U', '--netbios-name', 'IPA',
+            '-a', self.master.config.admin_password,
+            '--add-sids'
+        ])
+
+        # wait for replication to propagate the change on
+        # cn=adtrust agents,cn=sysaccounts,cn=etc,dc=ipa,dc=test
+        # as the ipa server-role-find call is using this entry to
+        # build its output
+        tasks.wait_for_replication(self.replicas[0].ldap_connect())
+
+        # double-check
+        assert self._ipa_replica_role_check(
+            self.replicas[0].hostname, self.serverroles['ADTC']
+        )
+        # check that master has no AD Trust-related role
+        assert not self._ipa_replica_role_check(
+            self.master.hostname, self.serverroles['ADTA']
+        )
+        self._check_rolecheck_backup_success(self.replicas[0])
+        self._check_rolecheck_backup_failure(self.master)
+
+        # make master become Trust Agent
+        cmd_input = (
+            # admin password:
+            self.master.config.admin_password + '\n' +
+            # WARNING: The smb.conf already exists. Running ipa-adtrust-install
+            # will break your existing samba configuration.
+            # Do you wish to continue? [no]:
+            'yes\n'
+            # Enable trusted domains support in slapi-nis? [no]:
+            '\n' +
+            # WARNING: 1 IPA masters are not yet able to serve information
+            # about users from trusted forests.
+            # Installer can add them to the list of IPA masters allowed to
+            # access information about trusts.
+            # If you choose to do so, you also need to restart LDAP service on
+            # those masters.
+            # Refer to ipa-adtrust-install(1) man page for details.
+            # IPA master[replica1.testrelm.test]?[no]:
+            'yes\n'
+        )
+        self.replicas[0].run_command([
+            'ipa-adtrust-install', '--add-agents'], stdin_text=cmd_input
+        )
+
+        # wait for replication to propagate the change on
+        # cn=adtrust agents,cn=sysaccounts,cn=etc,dc=ipa,dc=test
+        # as the ipa server-role-find call is using this entry to
+        # build its output
+        tasks.wait_for_replication(self.replicas[0].ldap_connect())
+
+        # check that master is now an AD Trust agent
+        assert self._ipa_replica_role_check(
+            self.master.hostname, self.serverroles['ADTA']
+        )
+        # but not an AD Trust controller
+        assert not self._ipa_replica_role_check(
+            self.master.hostname, self.serverroles['ADTC']
+        )
+        self._check_rolecheck_backup_success(self.replicas[0])
+        self._check_rolecheck_backup_failure(self.master)
+
+        # make master become Trust Controller
+        self.master.run_command([
+            'ipa-adtrust-install', '-U', '--netbios-name', 'IPA',
+            '-a', self.master.config.admin_password,
+            '--add-sids'
+        ])
+
+        # wait for replication to propagate the change on
+        # cn=adtrust agents,cn=sysaccounts,cn=etc,dc=ipa,dc=test
+        # as the ipa server-role-find call is using this entry to
+        # build its output
+        tasks.wait_for_replication(self.master.ldap_connect())
+
+        # master and replicas[0] are both AD Trust Controllers now.
+        for hostname in [self.master.hostname, self.replicas[0].hostname]:
+            assert self._ipa_replica_role_check(
+                hostname, self.serverroles['ADTC']
+            )
+        self._check_rolecheck_backup_success(self.master)
+        self._check_rolecheck_backup_success(self.replicas[0])
+
+        # switch replicas[0] to hidden
+        self.replicas[0].run_command([
+            'ipa', 'server-state',
+            self.replicas[0].hostname, '--state=hidden'
+        ])
+        self._check_rolecheck_backup_success(self.master)
+        self._check_rolecheck_backup_success(self.replicas[0])
