@@ -2,9 +2,9 @@
 # Copyright (C) 2016  FreeIPA Contributors see COPYING for license
 #
 import logging
+import re
 import time
 import pytest
-import six
 import dns.resolver
 import dns.rrset
 import dns.rdatatype
@@ -12,7 +12,7 @@ import dns.rdataclass
 
 from ipatests.test_integration.base import IntegrationTest
 from ipatests.pytest_ipa.integration import tasks
-from ipapython.dnsutil import DNSName
+from ipapython.dnsutil import DNSName, DNSResolver
 from ipalib.constants import IPA_CA_RECORD
 
 logger = logging.getLogger(__name__)
@@ -28,6 +28,17 @@ IPA_DEFAULT_MASTER_SRV_REC = (
     (DNSName(u'_kpasswd._udp'), 464),
 )
 
+IPA_DEFAULT_MASTER_URI_REC = (
+    (
+        DNSName('_kerberos'),
+        ("krb5srv:m:tcp:{hostname}", "krb5srv:m:udp:{hostname}")
+    ),
+    (
+        DNSName('_kpasswd'),
+        ("krb5srv:m:tcp:{hostname}", "krb5srv:m:udp:{hostname}")
+    ),
+)
+
 IPA_DEFAULT_ADTRUST_SRV_REC = (
     # srv record name, port
     (DNSName(u'_ldap._tcp.Default-First-Site-Name._sites.dc._msdcs'), 389),
@@ -39,20 +50,20 @@ IPA_DEFAULT_ADTRUST_SRV_REC = (
 )
 
 IPA_CA_A_REC = (
-    (DNSName(six.text_type(IPA_CA_RECORD))),
+    (DNSName(str(IPA_CA_RECORD))),
 )
 
 
 def resolve_records_from_server(rname, rtype, nameserver):
     error = None
-    res = dns.resolver.Resolver()
+    res = DNSResolver()
     res.nameservers = [nameserver]
     res.lifetime = 30
     logger.info("Query: %s %s, nameserver %s", rname, rtype, nameserver)
     # lets try to query 3x
     for _i in range(3):
         try:
-            ans = res.query(rname, rtype)
+            ans = res.resolve(rname, rtype)
             logger.info("Answer: %s", ans.rrset)
             return ans.rrset
         except (dns.resolver.NXDOMAIN, dns.resolver.Timeout) as e:
@@ -61,6 +72,7 @@ def resolve_records_from_server(rname, rtype, nameserver):
 
     pytest.fail("Query: {} {}, nameserver {} failed due to {}".format(
         rname, rtype, nameserver, error))
+    return None
 
 
 def _gen_expected_srv_rrset(rname, port, servers, ttl=86400):
@@ -78,11 +90,31 @@ def _gen_expected_srv_rrset(rname, port, servers, ttl=86400):
     )
 
 
+def _gen_expected_uri_rrset(rname, uri_templates, servers, ttl=86400):
+    rdata_list = [
+        "{prio} {weight} {uri}".format(
+            prio=prio,
+            weight=weight,
+            uri=uri_template.format(hostname=servername.make_absolute()),
+        )
+        for uri_template in uri_templates
+        for prio, weight, servername in servers
+    ]
+    return dns.rrset.from_text_list(
+        rname, ttl, dns.rdataclass.IN, dns.rdatatype.URI, rdata_list
+    )
+
+
 def _gen_expected_a_rrset(rname, servers, ttl=86400):
     return dns.rrset.from_text_list(rname, ttl, dns.rdataclass.IN,
                                     dns.rdatatype.A, servers)
 
 
+def _get_relative_weights(text):
+    """Takes location-show output and returns a list of percentages"""
+    return re.findall(r"\d+.\d%", text)
+
+
 class TestDNSLocations(IntegrationTest):
     """Simple test if SRV DNS records for IPA locations are generated properly
 
@@ -168,6 +200,20 @@ class TestDNSLocations(IntegrationTest):
                 "with IP: '{}' for name '{}' (expected:\n{}\ngot:\n{})".
                 format(server_ip, name_abs, expected, query))
 
+    def _test_URI_rec_against_server(self, server_ip, domain, expected_servers,
+                                     rec_list=IPA_DEFAULT_MASTER_URI_REC):
+        for rname, uri_templates in rec_list:
+            name_abs = rname.derelativize(domain)
+            expected = _gen_expected_uri_rrset(
+                name_abs, uri_templates, expected_servers)
+            query = resolve_records_from_server(
+                name_abs, 'URI', server_ip)
+
+            assert expected == query, (
+                "Expected and received DNS data do not match on server "
+                "with IP: '{}' for name '{}' (expected:\n{}\ngot:\n{})".
+                format(server_ip, name_abs, expected, query))
+
     def test_without_locations(self):
         """Servers are not in locations, this tests if basic system records
         are generated properly"""
@@ -179,6 +225,9 @@ class TestDNSLocations(IntegrationTest):
         for ip in (self.master.ip, self.replicas[0].ip, self.replicas[1].ip):
             self._test_SRV_rec_against_server(ip, self.domain,
                                               expected_servers)
+            self._test_URI_rec_against_server(
+                ip, self.domain, expected_servers
+            )
 
     def test_nsupdate_without_locations(self):
         """Test nsupdate file generated by dns-update-system-records
@@ -221,11 +270,19 @@ class TestDNSLocations(IntegrationTest):
         )
 
         self._test_SRV_rec_against_server(
-            self.replicas[0].ip, domain_prague_loc, servers_prague_loc)
+            self.replicas[0].ip, domain_prague_loc, servers_prague_loc
+        )
+        self._test_URI_rec_against_server(
+            self.replicas[0].ip, domain_prague_loc, servers_prague_loc
+        )
 
         for ip in (self.master.ip, self.replicas[1].ip):
             self._test_SRV_rec_against_server(
-                ip, domain_without_loc, servers_without_loc)
+                ip, domain_without_loc, servers_without_loc
+            )
+            self._test_URI_rec_against_server(
+                ip, domain_without_loc, servers_without_loc
+            )
 
     def test_two_replicas_in_location(self):
         """Put second replica to location and test if records changed properly
@@ -264,13 +321,25 @@ class TestDNSLocations(IntegrationTest):
                 self.master.domain.name).make_absolute())
 
         self._test_SRV_rec_against_server(
-            self.replicas[0].ip, domain_prague_loc, servers_prague_loc)
+            self.replicas[0].ip, domain_prague_loc, servers_prague_loc
+        )
+        self._test_URI_rec_against_server(
+            self.replicas[0].ip, domain_prague_loc, servers_prague_loc
+        )
 
         self._test_SRV_rec_against_server(
-            self.replicas[1].ip, domain_paris_loc, servers_paris_loc)
+            self.replicas[1].ip, domain_paris_loc, servers_paris_loc
+        )
+        self._test_URI_rec_against_server(
+            self.replicas[1].ip, domain_paris_loc, servers_paris_loc
+        )
 
         self._test_SRV_rec_against_server(
-            self.master.ip, domain_without_loc, servers_without_loc)
+            self.master.ip, domain_without_loc, servers_without_loc
+        )
+        self._test_URI_rec_against_server(
+            self.master.ip, domain_without_loc, servers_without_loc
+        )
 
     def test_all_servers_in_location(self):
         """Put master (as second server) to location and test if records
@@ -302,11 +371,19 @@ class TestDNSLocations(IntegrationTest):
                 self.master.domain.name).make_absolute())
 
         self._test_SRV_rec_against_server(
-            self.replicas[0].ip, domain_prague_loc, servers_prague_loc)
+            self.replicas[0].ip, domain_prague_loc, servers_prague_loc
+        )
+        self._test_URI_rec_against_server(
+            self.replicas[0].ip, domain_prague_loc, servers_prague_loc
+        )
 
         for ip in (self.replicas[1].ip, self.master.ip):
-            self._test_SRV_rec_against_server(ip, domain_paris_loc,
-                                              servers_paris_loc)
+            self._test_SRV_rec_against_server(
+                ip, domain_paris_loc, servers_paris_loc
+            )
+            self._test_URI_rec_against_server(
+                ip, domain_paris_loc, servers_paris_loc
+            )
 
     def test_change_weight(self):
         """Change weight of master and test if records changed properly
@@ -341,11 +418,94 @@ class TestDNSLocations(IntegrationTest):
                 self.master.domain.name).make_absolute())
 
         self._test_SRV_rec_against_server(
-            self.replicas[0].ip, domain_prague_loc, servers_prague_loc)
+            self.replicas[0].ip, domain_prague_loc, servers_prague_loc
+        )
+        self._test_URI_rec_against_server(
+            self.replicas[0].ip, domain_prague_loc, servers_prague_loc
+        )
 
         for ip in (self.replicas[1].ip, self.master.ip):
-            self._test_SRV_rec_against_server(ip, domain_paris_loc,
-                                              servers_paris_loc)
+            self._test_SRV_rec_against_server(
+                ip, domain_paris_loc, servers_paris_loc
+            )
+            self._test_URI_rec_against_server(
+                ip, domain_paris_loc, servers_paris_loc
+            )
+
+    def test_change_weight_relative_zero_0(self):
+        """Change weight of one master and check on relative weight %
+        """
+
+        new_weight = 0
+
+        # Put all servers into one location
+        self.master.run_command([
+            'ipa', 'server-mod', self.replicas[0].hostname, '--location',
+            self.LOC_PARIS])
+
+        # Modify one to have a weight of 0
+        result = self.master.run_command([
+            'ipa', 'server-mod', self.master.hostname, '--service-weight',
+            str(new_weight)
+        ])
+
+        result = self.master.run_command([
+            'ipa', 'location-show', self.LOC_PARIS
+        ])
+        weights = _get_relative_weights(result.stdout_text)
+        assert weights.count('0.1%') == 1
+        assert weights.count('50.0%') == 2
+
+    # The following three tests are name-sensitive so they run in
+    # a specific order. They use the paris location and depend on
+    # the existing values of the server location and weight to work
+    # properly
+    def test_change_weight_relative_zero_1(self):
+        """Change all weights to zero and ensure no div by zero
+        """
+
+        new_weight = 0
+
+        # Depends on order of test execution but all masters are now
+        # in LOC_PARIS and self.master has a weight of 0.
+
+        # Modify all replicas to have a weight of 0
+        for hostname in (self.replicas[0].hostname, self.replicas[1].hostname):
+            self.master.run_command([
+                'ipa', 'server-mod', hostname, '--service-weight',
+                str(new_weight)
+            ])
+
+        result = self.master.run_command([
+            'ipa', 'location-show', self.LOC_PARIS
+        ])
+        weights = _get_relative_weights(result.stdout_text)
+        assert weights.count('33.3%') == 3
+
+    def test_change_weight_relative_zero_2(self):
+        """Change to mixed weight values and check percentages
+        """
+
+        new_weight = 100
+
+        # Change master to be primary, replicas secondary
+        self.master.run_command([
+            'ipa', 'server-mod', self.master.hostname, '--service-weight',
+            '200'
+        ])
+        for hostname in (self.replicas[0].hostname,
+                         self.replicas[1].hostname):
+            self.master.run_command([
+                'ipa', 'server-mod', hostname, '--service-weight',
+                str(new_weight)
+            ])
+
+        result = self.master.run_command([
+            'ipa', 'location-show', self.LOC_PARIS
+        ])
+        weights = _get_relative_weights(result.stdout_text)
+        assert weights.count('50.0%') == 1
+        assert weights.count('25.0%') == 2
 
     def test_restore_locations_and_weight(self):
         """Restore locations and weight. Not just for test purposes but also
