@@ -21,8 +21,8 @@
 Test the `ipalib.cli` module.
 """
 
-from ipatests.util import raises, get_api, ClassChecker
-from ipalib import cli, plugable, frontend, backend
+from ipatests.util import raises, ClassChecker
+from ipalib import cli, plugable
 
 import pytest
 
@@ -41,10 +41,10 @@ class test_textui(ClassChecker):
         assert str(e) == 'rows: need %r or %r; got %r' % (list, tuple, 'hello')
         rows = [
             'hello',
-            'naughty',
+            'empathetic',
             'nurse',
         ]
-        assert o.max_col_width(rows) == len('naughty')
+        assert o.max_col_width(rows) == len('empathetic')
         rows = (
             ( 'a',  'bbb',  'ccccc'),
             ('aa', 'bbbb', 'cccccc'),
@@ -76,20 +76,20 @@ def get_cmd_name(i):
     return 'cmd_%d' % i
 
 
-class DummyCommand(object):
+class DummyCommand:
     def __init__(self, name):
         self.__name = name
 
-    def __get_name(self):
+    def __get_name(self):  # pylint: disable=unused-private-member, #4756
         return self.__name
     name = property(__get_name)
 
 
-class DummyAPI(object):
+class DummyAPI:
     def __init__(self, cnt):
-        self.__cmd = plugable.NameSpace(self.__cmd_iter(cnt))
+        self.__cmd = plugable.APINameSpace(self.__cmd_iter(cnt), DummyCommand)
 
-    def __get_cmd(self):
+    def __get_cmd(self):  # pylint: disable=unused-private-member, #4756
         return self.__cmd
     Command = property(__get_cmd)
 
