@@ -192,7 +192,7 @@ class UI_driver:
         if not NO_YAML and os.path.isfile(path):
             try:
                 with open(path, 'r') as conf:
-                    cls.config = yaml.load(conf)
+                    cls.config = yaml.safe_load(stream=conf)
             except yaml.YAMLError as e:
                 pytest.skip("Invalid Web UI config.\n%s" % e)
             except IOError as e:
@@ -505,6 +505,10 @@ class UI_driver:
         self.driver.get(self.get_url(entity, facet))
         self.wait_for_request(n=3, d=0.4)
 
+    def navigate_to_page(self, page):
+        self.driver.get('/'.join([self.get_base_url(), '#', 'p', page]))
+        self.wait_for_request(n=3, d=0.4)
+
     def navigate_by_menu(self, item, complete=True):
         """
         Navigate by using menu
@@ -1053,7 +1057,7 @@ class UI_driver:
 
         # Chrome does not close search area on click
         if list_cnt.is_displayed():
-            self.driver.switch_to_active_element().send_keys(Keys.RETURN)
+            self.driver.switch_to.active_element.send_keys(Keys.RETURN)
 
         self.wait()
 
@@ -1147,6 +1151,34 @@ class UI_driver:
                 return row
         return None
 
+    def get_row_by_column_value(self, key, column_name, parent=None,
+                                table_name=None):
+        """
+        Get the first matched row element of a search table with given key
+        matched against selected column. None if not found
+        """
+        rows = self.get_rows(parent, table_name)
+        s = "td div[name='%s']" % column_name
+        for row in rows:
+            has = self.find(s, By.CSS_SELECTOR, row)
+            if has.text == key:
+                return row
+        return None
+
+    def get_record_pkey(self, key, column, parent=None, table_name=None):
+        """
+        Get record pkey if value of column is known
+        """
+        row = self.get_row_by_column_value(key,
+                                           column_name=column,
+                                           parent=parent,
+                                           table_name=table_name)
+        val = None
+        if row:
+            el = self.find("td input", By.CSS_SELECTOR, row)
+            val = el.get_attribute("value")
+        return val
+
     def navigate_to_row_record(self, row, pkey_column=None):
         """
         Navigate to record by clicking on a link.
@@ -1437,10 +1469,6 @@ class UI_driver:
 
         last_element = data[len(data) - 1]
 
-        pkeys = []
-
-        for record in data:
-            pkeys.append(record['pkey'])
         if navigate:
             self.navigate_to_entity(entity, facet)
 
@@ -2301,10 +2329,10 @@ class UI_driver:
         notification_type = 'div.notification-area .alert-{}'.format(type)
         # wait for a half sec for notification to appear
         self.wait(0.5)
-        is_present = self.find(notification_type, By.CSS_SELECTOR)
+        is_present = self.find(notification_type, By.CSS_SELECTOR, many=True)
         assert is_present, "Notification not present"
         if assert_text:
-            assert assert_text in is_present.text
+            assert any(map(lambda x: assert_text in x.text, is_present))
 
     def assert_last_error_dialog(self, expected_err, details=False,
                                  dialog_name='error_dialog'):
@@ -2346,3 +2374,36 @@ class UI_driver:
             else:
                 assert value in checked_values, ('{} NOT checked while it '
                                                  'should be'.format(value))
+
+    def add_cert_to_record(self, pem_certs, pkey, entity='user',
+                           navigate=False, save=True):
+        """
+        Add certificate to particular user
+        pem_certs (list): base64/pem certificate(s)
+        pkey (str): user/host to add the certificate to
+        entity (str): name of entity where to navigate if navigate=True
+        navigate (bool): whether we should navigate to record
+        save (bool): whether we should click save after adding a key
+        """
+
+        if type(pem_certs) is not list:
+            pem_certs = [pem_certs]
+
+        if navigate:
+            self.navigate_to_entity(entity)
+            self.navigate_to_record(pkey)
+
+        for cert in pem_certs:
+            c_add = 'div[name="usercertificate"] button[name="add"]'
+            cert_add_btn = self.find(c_add, By.CSS_SELECTOR, strict=True)
+            cert_add_btn.click()
+            self.wait()
+            c_text_area = 'textarea.form-control'
+            text_area = self.find(c_text_area, By.CSS_SELECTOR, strict=True)
+            text_area.send_keys(cert)
+            self.wait()
+            self.dialog_button_click('ok')
+
+        # sometimes we do not want to save e.g. in order to test undo buttons
+        if save:
+            self.facet_button_click('save')
