@@ -19,24 +19,21 @@
 """
 Test the `ipaserver/plugins/cert.py` module against a RA.
 """
-from __future__ import print_function
+from __future__ import print_function, absolute_import
 
 import base64
-import nose
 import os
+
 import pytest
-import shutil
 import six
-import tempfile
 from ipalib import api
 from ipalib import errors
-from ipalib import x509
 from ipaplatform.paths import paths
-from ipapython import ipautil
+from ipapython.certdb import NSSDatabase
 from ipapython.dn import DN
 from ipapython.ipautil import run
+from ipatests.test_xmlrpc.testcert import subject_base
 from ipatests.test_xmlrpc.xmlrpc_test import XMLRPC_test
-from nose.tools import raises, assert_raises
 
 if six.PY3:
     unicode = str
@@ -58,8 +55,8 @@ def is_db_configured():
     aliasdir = api.env.dot_ipa + os.sep + 'alias' + os.sep + '.pwd'
 
     if (api.env.xmlrpc_uri == u'http://localhost:8888/ipa/xml' and
-       not ipautil.file_exists(aliasdir)):
-        raise nose.SkipTest('developer CA not configured in %s' % aliasdir)
+       not os.path.isfile(aliasdir)):
+        pytest.skip('developer CA not configured in %s' % aliasdir)
 
 # Test setup
 #
@@ -70,74 +67,57 @@ def is_db_configured():
 #
 # To test against Dogtag CA in the lite-server:
 #
-# - Copy the 3 NSS db files from /etc/httpd/alias to ~/.ipa/alias
-# - Copy /etc/httpd/alias/pwdfile.txt to  ~/.ipa/alias/.pwd.
+# - Copy the 3 NSS db files from /var/lib/ipa/radb to ~/.ipa/alias
+# - Copy /var/lib/ipa/radb/pwdfile.txt to  ~/.ipa/alias/.pwd.
 # - Change ownership of these files to be readable by you.
 #
 # The API tested depends on the value of ~/.ipa/default/ra_plugin when
 # running as the lite-server.
 
-
 class BaseCert(XMLRPC_test):
+    host_fqdn = u'ipatestcert.%s' % api.env.domain
+    service_princ = u'test/%s@%s' % (host_fqdn, api.env.realm)
+    certfile = None
+    nssdb = None
+    reqfile = None
+    subject = None
 
-    @classmethod
-    def setup_class(cls):
-        super(BaseCert, cls).setup_class()
-
+    @pytest.fixture(autouse=True, scope="class")
+    def basecert_setup(self, request, xmlrpc_setup):
         if 'cert_request' not in api.Command:
-            raise nose.SkipTest('cert_request not registered')
+            pytest.skip('cert_request not registered')
         if 'cert_show' not in api.Command:
-            raise nose.SkipTest('cert_show not registered')
+            pytest.skip('cert_show not registered')
 
         is_db_configured()
 
-    def run_certutil(self, args, stdin=None):
-        new_args = [paths.CERTUTIL, "-d", self.reqdir]
-        new_args = new_args + args
-        return ipautil.run(new_args, stdin)
-
-    def setup(self):
-        self.reqdir = tempfile.mkdtemp(prefix = "tmp-")
-        self.reqfile = self.reqdir + "/test.csr"
-        self.pwname = self.reqdir + "/pwd"
-        self.certfile = self.reqdir + "/cert.crt"
-
-        # Create an empty password file
-        fp = open(self.pwname, "w")
-        fp.write("\n")
-        fp.close()
-
+    @pytest.fixture(autouse=True)
+    def basecert_fsetup(self, request):
+        self.nssdb = NSSDatabase()
+        secdir = self.nssdb.secdir
+        self.reqfile = os.path.join(secdir, "test.csr")
+        self.certfile = os.path.join(secdir, "cert.crt")
         # Create our temporary NSS database
-        self.run_certutil(["-N", "-f", self.pwname])
+        self.nssdb.create_db()
+        self.subject = DN(('CN', self.host_fqdn), subject_base())
 
-        self.subject = DN(('CN', self.host_fqdn), x509.subject_base())
-
-    def teardown(self):
-        shutil.rmtree(self.reqdir, ignore_errors=True)
+        def fin():
+            self.nssdb.close()
+        request.addfinalizer(fin)
 
     def generateCSR(self, subject):
-        self.run_certutil(["-R", "-s", subject,
-                           "-o", self.reqfile,
-                           "-z", paths.GROUP,
-                           "-f", self.pwname,
-                           "-a",
-                           ])
-        fp = open(self.reqfile, "r")
-        data = fp.read()
-        fp.close()
-        return data
-
-    host_fqdn = u'ipatestcert.%s' % api.env.domain
-    service_princ = u'test/%s@%s' % (host_fqdn, api.env.realm)
+        self.nssdb.run_certutil([
+            "-R", "-s", subject,
+            "-o", self.reqfile,
+            "-z", paths.GROUP,
+            "-a",
+        ])
+        with open(self.reqfile, "rb") as f:
+            return f.read().decode('ascii')
 
 
 @pytest.mark.tier1
 class test_cert(BaseCert):
-
-    @classmethod
-    def setup_class(cls):
-        super(test_cert, cls).setup_class()
-
     """
     Test the `cert` plugin.
     """
@@ -149,11 +129,11 @@ class test_cert(BaseCert):
         This should fail because the service principal doesn't exist
         """
         # First create the host that will use this policy
-        res = api.Command['host_add'](self.host_fqdn, force= True)['result']
+        assert 'result' in api.Command['host_add'](self.host_fqdn, force=True)
 
-        csr = unicode(self.generateCSR(str(self.subject)))
-        with assert_raises(errors.NotFound):
-            res = api.Command['cert_request'](csr, principal=self.service_princ)
+        csr = self.generateCSR(str(self.subject))
+        with pytest.raises(errors.NotFound):
+            api.Command['cert_request'](csr, principal=self.service_princ)
 
     def test_0002_cert_add(self):
         """
@@ -162,7 +142,7 @@ class test_cert(BaseCert):
         # Our host should exist from previous test
         global cert, sn
 
-        csr = unicode(self.generateCSR(str(self.subject)))
+        csr = self.generateCSR(str(self.subject))
         res = api.Command['cert_request'](csr, principal=self.service_princ, add=True)['result']
         assert DN(res['subject']) == self.subject
         assert 'cacn' in res
@@ -175,8 +155,6 @@ class test_cert(BaseCert):
         """
         Verify that service-show has the right certificate using service-show.
         """
-        global cert
-
         res = api.Command['service_show'](self.service_princ)['result']
         assert base64.b64encode(res['usercertificate'][0]) == cert
 
@@ -184,8 +162,6 @@ class test_cert(BaseCert):
         """
         Verify that service-find has the right certificate using service-find.
         """
-        global cert
-
         # Assume there is only one service
         res = api.Command['service_find'](self.service_princ)['result']
         assert base64.b64encode(res[0]['usercertificate'][0]) == cert
@@ -196,9 +172,9 @@ class test_cert(BaseCert):
         See https://fedorahosted.org/freeipa/ticket/5881
         """
         result = api.Command.cert_show(sn, out=unicode(self.certfile))
-        with open(self.certfile, "r") as f:
-            pem_cert = unicode(f.read())
-        result = run(['openssl', 'x509', '-text'],
+        with open(self.certfile, "rb") as f:
+            pem_cert = f.read().decode('ascii')
+        result = run([paths.OPENSSL, 'x509', '-text'],
                      stdin=pem_cert, capture_output=True)
         assert _EXP_CRL_URI in result.output
         assert _EXP_OCSP_URI in result.output
@@ -209,7 +185,7 @@ class test_cert(BaseCert):
         """
         global newcert
 
-        csr = unicode(self.generateCSR(str(self.subject)))
+        csr = self.generateCSR(str(self.subject))
         res = api.Command['cert_request'](csr, principal=self.service_princ)['result']
         assert DN(res['subject']) == self.subject
         # save the cert for the service_show/find tests
@@ -219,8 +195,6 @@ class test_cert(BaseCert):
         """
         Verify the new certificate with service-show.
         """
-        global cert, newcert
-
         res = api.Command['service_show'](self.service_princ)['result']
 
         # Both the old and the new certs should be listed as certificates now
@@ -248,7 +222,39 @@ class test_cert(BaseCert):
         assert 'valid_not_before' in res
         assert 'valid_not_after' in res
 
-    def test_00010_cleanup(self):
+    def test_00010_san_in_cert(self):
+        """
+        Test if SAN extension is automatically added with default profile.
+        """
+        csr = self.generateCSR(str(self.subject))
+        res = api.Command[
+            'cert_request'](csr, principal=self.service_princ)['result']
+        assert 'san_dnsname' in res
+
+    def test_00011_emails_are_valid(self):
+        """
+        Verify the different scenarios when checking if any email addr
+        from DN or SAN extension does not appear in ldap entry.
+        """
+
+        from ipaserver.plugins.cert import _emails_are_valid
+        email_addrs = [u'any@EmAiL.CoM']
+        result = _emails_are_valid(email_addrs, [u'any@email.com'])
+        assert result
+
+        email_addrs = [u'any@EmAiL.CoM']
+        result = _emails_are_valid(email_addrs, [u'any@email.com',
+                                                 u'another@email.com'])
+        assert result
+
+        result = _emails_are_valid([], [u'any@email.com'])
+        assert result
+
+        email_addrs = [u'invalidEmailAddress']
+        result = _emails_are_valid(email_addrs, [])
+        assert not result
+
+    def test_99999_cleanup(self):
         """
         Clean up cert test data
         """
@@ -262,23 +268,20 @@ class test_cert(BaseCert):
 
 @pytest.mark.tier1
 class test_cert_find(XMLRPC_test):
-
-    @classmethod
-    def setup_class(cls):
-        super(test_cert_find, cls).setup_class()
-
-        if 'cert_find' not in api.Command:
-            raise nose.SkipTest('cert_find not registered')
-
-        if api.env.ra_plugin != 'dogtag':
-            raise nose.SkipTest('cert_find for dogtag CA only')
-
-        is_db_configured()
-
     """
     Test the `cert-find` command.
     """
-    short = api.env.host.split('.')[0]
+    @pytest.fixture(autouse=True, scope="class")
+    def certfind_setup(self, request, xmlrpc_setup):
+        if 'cert_find' not in api.Command:
+            pytest.skip('cert_find not registered')
+
+        if api.env.ra_plugin != 'dogtag':
+            pytest.skip('cert_find for dogtag CA only')
+
+        is_db_configured()
+
+    short = api.env.host.split('.', maxsplit=1)[0]
 
     def test_0001_find_all(self):
         """
@@ -302,6 +305,8 @@ class test_cert_find(XMLRPC_test):
         Search for the OCSP certificate.
         """
         res = api.Command['cert_find'](subject=u'OCSP Subsystem')
+        assert 'count' in res
+        assert res['count'], "No OSCP certificate found"
 
     def test_0004_find_this_host(self):
         """
@@ -414,12 +419,12 @@ class test_cert_find(XMLRPC_test):
         res = api.Command['cert_find'](sizelimit=0)
         assert 'count' in res and res['count'] == count_all
 
-    @raises(errors.ValidationError)
     def test_0028_find_negative_size(self):
         """
         Search with a negative sizelimit
         """
-        res = api.Command['cert_find'](sizelimit=-100)
+        with pytest.raises(errors.ValidationError):
+            api.Command['cert_find'](sizelimit=-100)
 
     def test_0029_search_for_notfound(self):
         """
@@ -435,28 +440,24 @@ class test_cert_find(XMLRPC_test):
         res = api.Command['cert_find'](subject=u'ipatestcert.%s' % api.env.domain)
         assert 'count' in res and res['count'] >= 1
 
-    @raises(errors.ConversionError)
     def test_0031_search_on_invalid_date(self):
         """
         Search using invalid date format
         """
-        res = api.Command['cert_find'](issuedon_from=u'xyz')
+        with pytest.raises(errors.ConversionError):
+            api.Command['cert_find'](issuedon_from=u'xyz')
 
 
 @pytest.mark.tier1
 class test_cert_revocation(BaseCert):
 
-    @classmethod
-    def setup_class(cls):
-        super(test_cert_revocation, cls).setup_class()
-
     # create CSR, request cert, revoke cert, check cert attributes
     def revoke_cert(self, reason):
         # add host
         assert 'result' in api.Command['host_add'](self.host_fqdn, force=True)
 
         # generate CSR, request certificate, obtain serial number
-        self.csr = unicode(self.generateCSR(str(self.subject)))
+        self.csr = self.generateCSR(str(self.subject))
         res = api.Command['cert_request'](self.csr,
                                           principal=self.service_princ,
                                           add=True, all=True)['result']
@@ -503,3 +504,41 @@ class test_cert_revocation(BaseCert):
 
     def test_revoke_with_reason_10(self):
         self.revoke_cert(10)
+
+
+@pytest.mark.tier1
+class test_cert_remove_hold(BaseCert):
+
+    # create CSR, request cert, revoke cert, remove hold
+    def test_revoke_and_remove_hold(self):
+        # add host
+        assert 'result' in api.Command['host_add'](self.host_fqdn, force=True)
+
+        # generate CSR, request certificate, obtain serial number
+        self.csr = self.generateCSR(str(self.subject))
+        res = api.Command['cert_request'](self.csr,
+                                          principal=self.service_princ,
+                                          add=True, all=True)['result']
+        serial_number = res['serial_number']
+
+        # revoke created certificate
+        assert 'result' in api.Command['cert_revoke'](
+            serial_number, revocation_reason=6)
+
+        # verify that certificate is revoked with correct reason
+        res2 = api.Command['cert_show'](serial_number, all=True)['result']
+        assert res2['revoked']
+        assert res2['revocation_reason'] == 6
+
+        # remove hold
+        res3 = api.Command['cert_remove_hold'](serial_number)['result']
+        assert res3['unrevoked']
+
+        # remove host
+        assert 'result' in api.Command['host_del'](self.host_fqdn)
+
+    def test_remove_hold_nonexistent_cert(self):
+        # remove hold must print 'Certificate ID xx not found'
+        with pytest.raises(errors.CertificateOperationError,
+                           match=r'Certificate ID 0x.* not found'):
+            api.Command['cert_remove_hold'](9999)
