@@ -36,7 +36,6 @@ from ipaplatform.paths import paths
 from ipapython import ipautil
 from ipapython.dn import DN
 from ipapython.dnsutil import DNSName
-from ipatests.test_util import yield_fixture
 from ipatests.test_xmlrpc import objectclasses
 from ipatests.test_xmlrpc.test_user_plugin import get_group_dn
 from ipatests.test_xmlrpc.testcert import get_testcert, subject_base
@@ -332,6 +331,12 @@ class TestCRUD(XMLRPC_test):
                 name='sshpubkey', error=u'options are not allowed')):
             command()
 
+    def test_update_shortname(self, host):
+        host.ensure_exists()
+        host.run_command('host_mod', host.shortname,
+                         updatedns=True,
+                         setattr='nshardwareplatform=linux')
+
     def test_delete_host(self, host):
         host.delete()
 
@@ -600,6 +605,27 @@ class TestProtectedMaster(XMLRPC_test):
                 error=u'An IPA master host cannot be deleted or disabled')):
             command()
 
+    def test_try_add_auth_ind_master(self, this_host):
+        command = this_host.make_update_command({
+            u'krbprincipalauthind': u'radius'})
+        with raises_exact(errors.ValidationError(
+            name='krbprincipalauthind',
+            error=u'authentication indicators not allowed '
+                'in service "host"'
+        )):
+            command()
+
+    def test_add_non_master_with_auth_ind(self, host5):
+        host5.ensure_missing()
+        command = host5.make_command(
+            'host_add', host5.fqdn, krbprincipalauthind=['radius'],
+            force=True
+        )
+        result = command()
+        # The fact that the command succeeds exercises the change but
+        # let's check the indicator as well.
+        assert result['result']['krbprincipalauthind'] == ('radius',)
+
 
 @pytest.mark.tier1
 class TestValidation(XMLRPC_test):
@@ -643,7 +669,7 @@ class TestValidation(XMLRPC_test):
         ), result)
 
 
-@yield_fixture
+@pytest.fixture
 def keytabname(request):
     keytabfd, keytabname = tempfile.mkstemp()
     try:
@@ -727,7 +753,7 @@ class TestHostFalsePwdChange(XMLRPC_test):
             command()
 
 
-@yield_fixture(scope='class')
+@pytest.fixture(scope='class')
 def dns_setup_nonameserver(host4):
     # Make sure that the server does not handle the reverse zone used
     # for the test
@@ -819,7 +845,7 @@ class TestHostNoNameserversForRevZone(XMLRPC_test):
                 pass
 
 
-@yield_fixture(scope='class')
+@pytest.fixture(scope='class')
 def dns_setup(host):
     try:
         host.run_command('dnszone_del', dnszone, revzone, revipv6zone,
