@@ -21,8 +21,11 @@ Test the `ipaserver/plugins/selinuxusermap.py` module.
 """
 
 from ipalib import api, errors
+from ipaplatform.constants import constants as platformconstants
+
 from ipatests.test_xmlrpc import objectclasses
 from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_digits,
+                                              fuzzy_set_optional_oc,
                                               fuzzy_uuid)
 from ipapython.dn import DN
 from ipatests.util import Fuzzy
@@ -30,8 +33,17 @@ from ipatests.test_xmlrpc.test_user_plugin import get_user_result
 import pytest
 
 rule1 = u'selinuxrule1'
-selinuxuser1 = u'guest_u:s0'
-selinuxuser2 = u'xguest_u:s0'
+selinux_users = platformconstants.SELINUX_USERMAP_ORDER.split("$")
+selinuxuser1 = selinux_users[0]
+selinuxuser2 = selinux_users[1]
+
+INVALID_MCS = "Invalid MCS value, must match {}, where max category {}".format(
+    platformconstants.SELINUX_MCS_REGEX,
+    platformconstants.SELINUX_MCS_MAX)
+
+INVALID_MLS = "Invalid MLS value, must match {}, where max level {}".format(
+    platformconstants.SELINUX_MLS_REGEX,
+    platformconstants.SELINUX_MLS_MAX)
 
 user1 = u'tuser1'
 group1 = u'testgroup1'
@@ -107,7 +119,7 @@ class test_selinuxusermap(Declarative):
                     ipaselinuxuser=[selinuxuser1],
                     objectclass=objectclasses.selinuxusermap,
                     ipauniqueid=[fuzzy_uuid],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                 ),
             ),
@@ -134,7 +146,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser1],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                 ),
             ),
@@ -151,7 +163,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                 ),
                 summary=u'Modified SELinux User Map "%s"' % rule1,
                 value=rule1,
@@ -167,7 +179,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                 ),
                 summary=None,
@@ -185,7 +197,7 @@ class test_selinuxusermap(Declarative):
                     dict(
                         cn=[rule1],
                         ipaselinuxuser=[selinuxuser2],
-                        ipaenabledflag=[u'TRUE'],
+                        ipaenabledflag=[True],
                         dn=fuzzy_selinuxusermapdn,
                     ),
                 ],
@@ -220,7 +232,8 @@ class test_selinuxusermap(Declarative):
                     cn=[group1],
                     description=[u'Test desc 1'],
                     gidnumber=[fuzzy_digits],
-                    objectclass=objectclasses.group + [u'posixgroup'],
+                    objectclass=fuzzy_set_optional_oc(
+                        objectclasses.posixgroup, 'ipantgroupattrs'),
                     ipauniqueid=[fuzzy_uuid],
                     dn=DN(('cn', group1), ('cn', 'groups'), ('cn', 'accounts'),
                           api.env.basedn),
@@ -241,6 +254,7 @@ class test_selinuxusermap(Declarative):
                         group=tuple(),
                         user=tuple(),
                         service=tuple(),
+                        idoverrideuser=tuple(),
                     ),
                 ),
                 result={
@@ -297,7 +311,7 @@ class test_selinuxusermap(Declarative):
                     objectclass=objectclasses.hbacrule,
                     ipauniqueid=[fuzzy_uuid],
                     accessruletype=[u'allow'],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_hbacruledn,
                 ),
             ),
@@ -317,7 +331,7 @@ class test_selinuxusermap(Declarative):
                     objectclass=objectclasses.hbacrule,
                     ipauniqueid=[fuzzy_uuid],
                     accessruletype=[u'allow'],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_hbacruledn,
                 ),
             ),
@@ -335,7 +349,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     memberuser_user=[user1],
                     dn=fuzzy_selinuxusermapdn,
                 ),
@@ -356,7 +370,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     memberuser_user=[user1],
                     dn=fuzzy_selinuxusermapdn,
                 ),
@@ -373,7 +387,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                 ),
             )
@@ -394,7 +408,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                 ),
             )
@@ -410,7 +424,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     memberuser_group=[group1],
                     dn=fuzzy_selinuxusermapdn,
                 ),
@@ -427,7 +441,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     memberhost_host=[host1],
                     memberuser_group=[group1],
                     dn=fuzzy_selinuxusermapdn,
@@ -494,7 +508,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     memberuser_group=[group1],
                     dn=fuzzy_selinuxusermapdn,
                 ),
@@ -512,7 +526,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                 ),
             )
@@ -538,7 +552,7 @@ class test_selinuxusermap(Declarative):
                 result=dict(
                     cn=[rule1],
                     ipaselinuxuser=[selinuxuser2],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     seealso=hbacrule1,
                 ),
                 summary=u'Modified SELinux User Map "%s"' % rule1,
@@ -623,44 +637,46 @@ class test_selinuxusermap(Declarative):
             ),
             expected=errors.ValidationError(
                 name='selinuxuser',
-                error=u'Invalid SELinux user name, only a-Z, _ '
-                      'and . are allowed'
+                error=u'Invalid SELinux user name, must match {}'.format(
+                    platformconstants.SELINUX_USER_REGEX)
             ),
         ),
 
 
         dict(
-            desc='Create rule with invalid MCS xguest_u:s999',
+            desc='Create rule with invalid MLS foo:s{}'.format(
+                platformconstants.SELINUX_MLS_MAX + 1),
             command=(
                 'selinuxusermap_add', [rule1],
-                     dict(ipaselinuxuser=u'xguest_u:s999')
+                dict(ipaselinuxuser=u'foo:s{}'.format(
+                    platformconstants.SELINUX_MLS_MAX + 1))
             ),
             expected=errors.ValidationError(name='selinuxuser',
-                error=u'Invalid MLS value, must match s[0-15](-s[0-15])'),
+                                            error=INVALID_MLS),
         ),
 
 
         dict(
-            desc='Create rule with invalid MLS xguest_u:s0:p88',
+            desc='Create rule with invalid MCS foo:s0:p88',
             command=(
                 'selinuxusermap_add', [rule1],
-                    dict(ipaselinuxuser=u'xguest_u:s0:p88')
+                dict(ipaselinuxuser=u'foo:s0:p88')
             ),
             expected=errors.ValidationError(name='selinuxuser',
-                error=u'Invalid MCS value, must match c[0-1023].c[0-1023] ' +
-                    u'and/or c[0-1023]-c[0-c0123]'),
+                                            error=INVALID_MCS),
         ),
 
 
         dict(
-            desc='Create rule with invalid MLS xguest_u:s0:c0.c1028',
+            desc='Create rule with invalid MCS foo:s0:c0.c{}'.format(
+                platformconstants.SELINUX_MCS_MAX + 1),
             command=(
                 'selinuxusermap_add', [rule1],
-                    dict(ipaselinuxuser=u'xguest_u:s0-s0:c0.c1028')
+                dict(ipaselinuxuser=u'foo:s0-s0:c0.c{}'.format(
+                    platformconstants.SELINUX_MCS_MAX + 1))
             ),
             expected=errors.ValidationError(name='selinuxuser',
-                error=u'Invalid MCS value, must match c[0-1023].c[0-1023] ' +
-                    u'and/or c[0-1023]-c[0-c0123]'),
+                                            error=INVALID_MCS),
         ),
 
 
@@ -671,7 +687,7 @@ class test_selinuxusermap(Declarative):
                     dict(setattr=u'ipaselinuxuser=deny')
             ),
             expected=errors.ValidationError(name='ipaselinuxuser',
-                error=u'Invalid MLS value, must match s[0-15](-s[0-15])'),
+                                            error=INVALID_MLS),
         ),
 
         dict(
@@ -738,7 +754,7 @@ class test_selinuxusermap(Declarative):
                     ipaselinuxuser=[selinuxuser1],
                     objectclass=objectclasses.selinuxusermap,
                     ipauniqueid=[fuzzy_uuid],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                     seealso=hbacrule1
                 ),
@@ -809,7 +825,7 @@ class test_selinuxusermap(Declarative):
                     ipaselinuxuser=[selinuxuser1],
                     objectclass=objectclasses.selinuxusermap,
                     ipauniqueid=[fuzzy_uuid],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                     usercategory=[u'all'],
                     hostcategory=[u'all']
@@ -850,7 +866,7 @@ class test_selinuxusermap(Declarative):
                     ipaselinuxuser=[selinuxuser1],
                     objectclass=objectclasses.selinuxusermap,
                     ipauniqueid=[fuzzy_uuid],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                 ),
             ),
@@ -895,7 +911,7 @@ class test_selinuxusermap(Declarative):
                     ipaselinuxuser=[selinuxuser1],
                     objectclass=objectclasses.selinuxusermap,
                     ipauniqueid=[fuzzy_uuid],
-                    ipaenabledflag=[u'TRUE'],
+                    ipaenabledflag=[True],
                     dn=fuzzy_selinuxusermapdn,
                     seealso=u'allow_all',
                 ),
