@@ -2,7 +2,7 @@
 # Copyright (C) 2015  FreeIPA Contributors see COPYING for license
 #
 """
-Test the `ipalib/plugins/serviceconstraint.py` module.
+Test the `ipaserver/plugins/serviceconstraint.py` module.
 """
 
 from ipalib import api, errors
@@ -17,6 +17,8 @@ target1 = u'test1-targets'
 target2 = u'test2-targets'
 princ1 = u'HTTP/%s@%s' % (api.env.host, api.env.realm)
 princ2 = u'ldap/%s@%s' % (api.env.host, api.env.realm)
+princ3 = u'host/%s@%s' % (api.env.host, api.env.realm)
+host3 = api.env.host
 
 
 def get_servicedelegation_dn(cn):
@@ -134,8 +136,8 @@ class test_servicedelegation(Declarative):
 
 
         dict(
-            desc='Search for all rules',
-            command=('servicedelegationrule_find', [], {}),
+            desc='Search for all rules with members',
+            command=('servicedelegationrule_find', [], {'no_members': False}),
             expected=dict(
                 summary=u'3 service delegation rules matched',
                 count=3,
@@ -162,6 +164,32 @@ class test_servicedelegation(Declarative):
         ),
 
 
+        dict(
+            desc='Search for all rules',
+            command=('servicedelegationrule_find', [], {}),
+            expected=dict(
+                summary=u'3 service delegation rules matched',
+                count=3,
+                truncated=False,
+                result=[
+                    {
+                        'dn': get_servicedelegation_dn(u'ipa-http-delegation'),
+                        'cn': [u'ipa-http-delegation'],
+                        'memberprincipal': [princ1],
+                    },
+                    dict(
+                        dn=get_servicedelegation_dn(rule2),
+                        cn=[rule2],
+                    ),
+                    dict(
+                        dn=get_servicedelegation_dn(rule1),
+                        cn=[rule1],
+                    ),
+                ],
+            ),
+        ),
+
+
         dict(
             desc='Create target %r' % target1,
             command=(
@@ -409,7 +437,7 @@ class test_servicedelegation(Declarative):
                     failed_memberprincipal=dict(
                         memberprincipal=[
                             [u'HTTP/notfound@%s' % api.env.realm,
-                             u'no such entry']
+                             u'no matching entry found']
                             ],
                     ),
                 ),
@@ -422,6 +450,50 @@ class test_servicedelegation(Declarative):
         ),
 
 
+        dict(
+            desc='Add host as a member %r to %r' % (host3, rule1),
+            command=(
+                'servicedelegationrule_add_member', [rule1],
+                dict(principal=princ3)
+            ),
+            expected=dict(
+                completed=1,
+                failed=dict(
+                    failed_memberprincipal=dict(
+                        memberprincipal=tuple(),
+                    ),
+                ),
+                result={
+                    'dn': get_servicedelegation_dn(rule1),
+                    'memberprincipal': (princ1, princ3),
+                    'cn': [rule1],
+                },
+            ),
+        ),
+
+
+        dict(
+            desc='Remove a host member %r from %r' % (host3, rule1),
+            command=(
+                'servicedelegationrule_remove_member', [rule1],
+                dict(principal=host3)
+            ),
+            expected=dict(
+                completed=1,
+                failed=dict(
+                    failed_memberprincipal=dict(
+                        memberprincipal=tuple(),
+                    ),
+                ),
+                result={
+                    'dn': get_servicedelegation_dn(rule1),
+                    'memberprincipal': (princ1,),
+                    'cn': [rule1],
+                },
+            ),
+        ),
+
+
         dict(
             desc='Remove a member %r from %r' % (princ1, rule1),
             command=(
@@ -530,7 +602,7 @@ class test_servicedelegation(Declarative):
                     failed_memberprincipal=dict(
                         memberprincipal=[
                             [u'HTTP/notfound@%s' % api.env.realm,
-                             u'no such entry']
+                             u'no matching entry found']
                         ],
                     ),
                 ),
